
# set directory where files are located
dir1 <- "D:\\Research\\Brandon\\Candidate Placements\\rep\\"

# load an r library, downloading if necesary
library0 <- function(s)
{
	if(s %in% rownames(installed.packages()) == FALSE) {install.packages(s,repos='http://cran.us.r-project.org')}
	require(s,character.only=TRUE)
}

# load libraries
library0("foreign") # for loading stata files
library0("fastmatch")
library0("Formula") # for formulas with multiple parts
library0("stringr")
library0("gmm") # for gmm estimation
library0("numDeriv") # for numerical derivatives
library0("AER") # for IV

# various functions
apply0 <- function(func1,x,subset=numeric(0),by=numeric(0),bys=numeric(0),w=logical(0),...)
{
	n <- length(x)
	if(length(subset) != 0 & length(subset) != n) stop("error in apply0 -- incompadible dimensions (subset)")
	if(length(by) != 0 & length(by) != n) stop("error in apply0 -- incompadible dimensions (by)")
	if(length(w) != 0 & length(w) != n) stop("error in apply0 -- incompadible dimensions (w)")

	if(length(subset) == 0) {
		subset1 <- rep(TRUE,n)
	} else {
		subset1 <- subset
	}
	subset1 <- subset1 & !is.na(x)
	if(length(w) != 0) subset1 <- subset1 & !is.na(w)
	if(length(by) != 0) by1 <- subset(by,subset1)
	x1 <- x[subset1]
	if(length(w) != 0) w1 <- w[subset1]
	if(length(x1) == 0) {
		NA
	} else {
		if(length(by) == 0) {
			if(length(w) == 0) {
				func1(x1,...)
			} else {
				func1(x1,w=w1,...)
			}
		} else {
			if(length(bys) == 0) bys <- sort(unique(by1))
			
			# there has to be a better way than this...
			if(length(w) == 0) {
				dat1 <- cbind(x1,by1)[order(by1),]
			} else {
				dat1 <- cbind(x1,w1,by1)[order(by1),]
			}
			J <- length(unique(by1))
			start <- rep(NA,J)
			start[1] <- 1
			tab1 <- table(by1)
			if(J > 1) {
				for(j in 2:J) start[j] <- start[j-1] + tab1[j-1]
				end <- c(start[2:J]-1,length(x1))
			} else {
				end <- length(x1)
			}
			ret <- rep(NA,length(bys))
			if(length(w) == 0) {
				for(j in 1:J) {
					k <- fmatch(dat1[start[j],2],bys)
					if(!is.na(k)) ret[k] <- func1(as.numeric(dat1[start[j]:end[j],1]),...)
				}
			} else {
				for(j in 1:J) {
					k <- fmatch(dat1[start[j],2],bys)
					if(!is.na(k)) ret[k] <- func1(as.numeric(dat1[start[j]:end[j],1]),...,w=as.numeric(dat1[start[j]:end[j],2]))
				}
			}
			names(ret) <- bys
			ret
		}
	}
}

mean0 <- function(x,w=numeric(0),subset=numeric(0),by=numeric(0),bys=numeric(0),se=FALSE,ci=FALSE)
{
	n <- length(x)
	if(length(subset) != 0 & length(subset) != n) stop("error in mean0 -- incompadible dimensions (subset)")
	if(length(w) != 0 & length(w) != n) stop("error in mean0 -- incompadible dimensions (w)")
	if(!se & !ci) {
		if(length(w) == 0) {
			apply0(mean,x,subset,by,bys)
		} else {
			if(length(subset) == 0) {
				subset1 <- !is.na(x) & !is.na(w)
			} else {
				subset1 <- !is.na(x) & !is.na(w) & subset
			}
			apply0(mean,w*x,subset1,by,bys) / apply0(mean,w,subset1,by,bys)
		}
	} else {
		ret <- list()
		if(length(w) == 0) {
			ret$est <- apply0(mean,x,subset,by,bys)
			se1 <- ret$se <- (var0(x,subset,by,bys) / n0(x,subset,by,bys))^.5
		} else {
			if(length(subset) == 0) {
				subset1 <- !is.na(x) & !is.na(w)
			} else {
				subset1 <- !is.na(x) & !is.na(w) & subset
			}
			W = apply0(sum,w,subset1,by,bys)
			ret <- list()
			ret$est <- apply0(sum,w*x,subset1,by,bys) / W 
			se1 <- (var0(w*x,subset1,by,bys) * n0(w*x,subset1,by,bys) / W^2)^.5
		}
		if(se) ret$se <- se1
		if(ci) ret$ci <- c(ret$est - 1.96 * se1,ret$est + 1.96 * se1)
		ret
	}
}

median0 <- function(x,subset=numeric(0),by=numeric(0),bys=numeric(0),se=FALSE,w=numeric(0))
{
	if(!se) {
		if(length(w) == 0) {
			apply0(median,x,subset,by,bys)
		} else {
			apply0(weighted.median,x,subset,by,bys,w=w)
		}
	} else {
		if(length(w) == 0) {
			list(est=apply0(median,x,subset,by,bys),se=apply0(medianse,x,subset,by,bys))
		} else {
			stop("error in median0 -- does not handle s.e. of weighted median yet")
		}
	}
}

quantile0 <- function(x,probs,subset=logical(0),by=numeric(0),bys=numeric(0),w=numeric(0))
{
	n <- length(x)
	if(length(subset) != 0 & length(subset) != n) stop("error in quantile0 -- incompadible dimensions (subset)")
	if(length(w) != 0 & length(w) != n) stop("error in quantile0 -- incompadible dimensions (w)")
	k <- length(probs)
	if(length(by) == 0) {
		m <- 1
	} else {
		if(length(bys) == 0) bys <- sort(unique(by))
		m <- length(bys)
	}
	ret <- matrix(rep(NA,m*k),nrow=m)
	if(length(w) == 0) {
		for(i in 1:k) ret[,i] <- apply0(quantile,x,subset,by,bys,probs=probs[i])
	} else {
		if(length(subset) == 0) subset1 <- rep(TRUE,n)
		subset1 <- subset & !is.na(w)
		for(i in 1:k) ret[,i] <- apply0(wtd.quantile0,x,subset1,by,bys,probs=probs[i],w=w)
	}
	rownames(ret) <- bys
	colnames(ret) <- probs
	ret
}
		
density0 <- function(x,bw=logical(0),subset=logical(0),w=logical(0))
{
	n <- length(x)
	if(length(subset) != 0 & length(subset) != n) stop(paste("error in density0 -- problem with subset\nn = ",n,"\nlength(subset) = ",length(subset),sep=""))
	if(length(w) != 0 & length(w) != n) stop("error in density0 -- problem with w")
	if(length(w) == 0) {
		if(length(subset) == 0) {
			x1 <- x[!is.na(x)]
		} else {
			x1 <- x[!is.na(x)&!is.na(subset)&subset]
		}
		if(length(bw) == 0) {
			density(x1)
		} else {
			density(x1,bw)
		}
	} else {
		if(length(subset) == 0) {
			x1 <- x[!is.na(x) & !is.na(w)]
			w1 <- w[!is.na(x) & !is.na(w)]
		} else {
			x1 <- x[!is.na(x)&!is.na(w)&!is.na(subset)&subset]
			w1 <- w[!is.na(x)&!is.na(w)&!is.na(subset)&subset]
		}
		w1 <- w1 / sum(w1)
		if(length(bw) == 0) {
			density(x1,weights=w1)
		} else {
			density(x1,bw,weights=w1)
		}
	}	
}

# vectorized function that determines if a row of a matrix has any missing values
rowmiss <- function(X)
{
	#!apply(X, 1, function(x)!any(is.na(x)))
	rowSums(is.na(X))>0 # faster?
}

# get the model response, including NAs, from a multipart formula
model.response0 <- function(f1,lhs=logical(0))
{
	mf <- model.frame(f1,na.action=na.pass)
	if(length(lhs) == 0) mr <- model.response(data=mf)
	else                 mr <- model.part(f1,data=mf,lhs=lhs)
	mr
}

# get the model matrix, including NAs, from a multipart formula
model.matrix0 <- function(f1,rhs=logical(0))
{
	mf <- model.frame(f1,na.action=na.pass)
	if(length(rhs) == 0) {
		mm <- model.matrix(f1,data=mf)
	} else {
		mm <- model.matrix(f1,data=mf,rhs=rhs)
	}
	mm
}

# get columns while keeping columns names with n x 1 matrix
getcols <- function(A,idx)
{
	B <- as.matrix(A[,idx])
	colnames(B) <- colnames(A)[idx]
	B
}

# get all the y's and X's from a multipart formula, eliminating NAs and subseting
formula.data <- function(f,subset=logical(0),list1=logical(0),se="",nointercept=FALSE)
{
	f1 <- Formula(f)
	len1 <- length(f1)
	if(length(len1) != 2) stop("error in formula.data -- invalid formula")
	y <- list()
	X <- list()
	for(i in 1:len1[1]) y[[i]] <- model.response0(f1,lhs=i)
	for(i in 1:len1[2]) X[[i]] <- model.matrix0(f1,rhs=i)
	ptm <- proc.time()
	y0 <- y
	X0 <- X
	n <- nrow(as.matrix(y[[1]]))
	drop <- rep(0,n)
	for(i in 1:len1[1]) drop <- drop | is.na(y[[i]])
	for(i in 1:len1[2]) drop <- drop | rowmiss(X[[i]])
	if(length(subset) == n) {
		drop <- drop | !subset
		drop[is.na(drop)] <- TRUE
	} else if(length(subset) != 0) {
		stop("error in formula data")
	}
	for(i in 1:len1[1]) y[[i]] <- y[[i]][!drop]
	for(i in 1:len1[2]) X[[i]] <- X[[i]][!drop,]
	if(nointercept) {
		for(i in 1:len1[2]) X[[i]] <- getcols(X[[i]],2:ncol(X[[i]]))
		for(i in 1:len1[2]) X0[[i]] <- getcols(X0[[i]],2:ncol(X0[[i]]))
	}
	if(length(list1) > 0) for(i in 1:length(list1)) if(length(list1[[i]]) > 0)
	{
		if(is.matrix(list1[[i]])) {
			if(nrow(list1[[i]]) != n) stop("error in formula.data -- element of list1 does not have n rows")
			list1[[i]] <- as.matrix(list1[[i]][!drop,])
		} else if(is.array(list1[[i]])) {
			if(length(dim(list1[[i]])) != 3) stop("error in formula.data -- array is not a 3D array")
			if(dim(list1[[i]])[1] != n) stop("error in formula.data -- element of list1 does not have n rows")
			list1[[i]] <- as.array(list1[[i]][!drop,,])
		} else {
			if(length(list1[[i]]) != n) stop("error in formula.data -- element of list1 is not of length n")
			list1[[i]] <- list1[[i]][!drop]
		}
	}

	for(i in 1:len1[2]) {
		keepcol <- colSums(abs(X[[i]]))!=0 # drop columns that are all zero (typically created by factor with subset
		X[[i]] <- getcols(X[[i]],(1:ncol(X[[i]]))[keepcol])
		X0[[i]] <- getcols(X0[[i]],(1:ncol(X0[[i]]))[keepcol])
	}
	list(y=y,X=X,y0=y0,X0=X0,list1=list1)
}

# function for robust standard errors
robust0 <- function(fm)
{
	require(sandwich, quietly = TRUE)
	require(lmtest, quietly = TRUE)
	N <- length(fm$residuals)
	M <- N
	K <- fm$rank
	dfc <- (M/(M-1))*((N-1)/(N-K))
	uj  <- apply(estfun(fm),2, function(x) tapply(x, 1:N, sum))
	vcovCL <- dfc*sandwich(fm, meat=crossprod(uj)/N)
	sm <- summary(fm)
	model <- coeftest(fm, vcovCL)
	list(model=model,N=N,r.squared=sm$r.squared,coef=sm$coefficients[1:K],theta=sm$coefficients[1:K],se=(diag(vcovCL))^.5,V=vcovCL)
}

# function for clustered standard errors
cluster0 <- function(fm, cl)
{
	require(sandwich, quietly = TRUE)
	require(lmtest, quietly = TRUE)
	not <- attr(fm$model,"na.action")
	if(!is.null(not)) { cl <- cl[-not] }
	M <- length(unique(cl))
	N <- length(cl)
	K <- fm$rank
	dfc <- (M/(M-1))*((N-1)/(N-K))
	uj <- apply(estfun(fm),2, function(x) tapply(x, cl, sum));
	vcovCL <- dfc*sandwich(fm, meat=crossprod(uj)/N)
	sm <- summary(fm)
	resid <- summary(fm)$residuals
	if(length(resid) > 0) {
		#sigmasqse <- (mean(((M / N) * aggregate((summary(fm)$residuals)^2,by=list(cl),FUN=sum)[2])^2)-(summary(fm)$sigma)^4)^.5 / M^.5
		sigmasqse <- NA
		model <- coeftest(fm, vcovCL)
		list(model=model,N=N,Clusters=M,r.squared=summary(fm)$r.squared,sigmasq=summary(fm)$sigma^2,sigmasqse=sigmasqse,coef=sm$coefficients[1:K],theta=sm$coefficients[1:K],se=diag(vcovCL)^.5,V=vcovCL)
	} else {
		model <- coeftest(fm, vcovCL)
		list(model=model,N=N,Clusters=M,r.squared=fm$r.squared,coef=fm$coefficients[1:K],theta=fm$coefficients[1:K],se=diag(vcovCL)^.5,V=vcovCL)
	}
}

get.ses <- function(m1,se,unitid=logical(0),timeid=logical(0),lag=logical(0))
{
	     if(se == "classic"                     ) stop("error in get.ses --- don't call with classic")
	else if(se == "cluster"                     ) res <- cluster0(m1,unitid)
	else if(se == "nw" | se == "neweywest"      ) res <- neweywest0(m1,timeid)
	else if(se == "pnw" | se == "panelneweywest") res <- panelneweywest0(m1,timeid,unitid,lag)
	else if(se == "robust")                       res <- robust0(m1) # the default option
	else                                          stop("invalid se option")
}

# linear regression, with subset, no intercept, and cluster options
lm0 <- function(f1,se="robust",w=logical(0),subset=logical(0),nointercept=FALSE,unitid=logical(0),timeid=logical(0),lag=logical(0))
{
	# get y and x
	dat <- formula.data(f1,subset,list(unitid=unitid,timeid=timeid,w=w),se=se,nointercept=nointercept)
	if(length(dat$y) != 1) stop("error in lm0 -- invalid number of repsonses")
	if(length(dat$X) != 1) stop("error in lm0 -- invalid RHS")
	y1 <- dat$y[[1]]
	x1 <- dat$X[[1]]
	w1 <- dat$list1$w
	unitid1 <- dat$list1$unitid
	timeid1 <- dat$list1$timeid
	
	# estimate the model
	if(length(w1) == 0) {
		lm1 <- lm(y1 ~ 0 + x1)
	} else {
		lm1 <- lm(y1 ~ 0 + x1,weights=w1)
	}
	if(se=="classic") {
		s1 <- summary(lm1)
		res <- list()
		res$coef <- s1$coefficients[,1]
		res$se <- s1$coefficients[,2]
		res$N <- length(y1)
		res$r.squared <- s1$r.squared
		res$ivs <- names(lm1$coefficients)
	} else if(se == "cluster-wild") {
		K <- lm1$rank # number of covariates
		p.val <- lm.cluster.wild(f1,logical(0),x1=x1,y1=y1,unitid=unitid1,forceNull=FALSE)
		res <- get.ses(lm1,se="cluster",unitid=unitid1,lag=lag)
		res$ivs <- rownames(res$model)
		res$p.val <- p.val	
	} else {
		res <- get.ses(lm1,se,timeid=timeid1,unitid=unitid1,lag=lag)
		res$ivs <- rownames(res$model)
	}

	if(length(unitid1) == 0) {
		res$Clusters <- NULL
	} else {
		res$Clusters <- length(unique(unitid1))
	}
	
	# model predictions
	x <- dat$X0[[1]]
	pred <- as.vector(x %*% lm1$coef)
	XpX <- t(x1) %*% x1
	res$pred <- pred
	res$res <- dat$y0[[1]][[1]] - pred
	res$XpX <- XpX

	# hack to get correct model fit when intercept is included as a variable
	if(length(w1) == 0) {
		lm2 <- lm(y1 ~ x1)
	} else {
		lm2 <- lm(y1 ~ x1,weights=w1)
	}

	res$theta <- res$coef
	res$r.squared <- summary(lm2)$r.squared
	res$sigma <- summary(lm2)$sigma
	res$AIC <- AIC(lm2)
	res$BIC <- BIC(lm2)
	fac <- (res$N-1)/(res$N-ncol(x))
	res$r.squared.adj <- fac * res$r.squared + 1 - fac
	res$ivs <- substr(res$ivs,3,nchar(res$ivs))
	res$ivs[res$ivs=="tercept)"] <- "Intercept"
	if(se=="classic") {
		res$V <- res$sigma^2 * solve(XpX)
		res$predse <- rep(NA,nrow(x))
		for(n in 1:nrow(x)) res$predse[n] <- res$sigma * (1 + t(x[n,]) %*% solve(XpX) %*% x[n,])^.5
		tcrit <- qt(.975,res$N-ncol(x))
		res$predlow <- res$pred - tcrit * res$predse
		res$predhigh <- res$pred + tcrit * res$predse
	}
	res
}

# IV regression, with subset and cluster options
ivreg0 <- function(f1,se="robust",subset=logical(0),unitid=logical(0),timeid=logical(0),liml=FALSE,lag=logical(0),w=logical(0))
{
	# get y, X, and Z
	dat <- formula.data(f1,subset,list(unitid=unitid,timeid=timeid,w=w),se=se,nointercept=TRUE)
	if(length(dat$y) != 1) stop("error in ivreg0 -- invalid number of repsonses")
	if(length(dat$X) != 2) stop("error in ivreg0 -- invalid RHS")
	y <- dat$y[[1]]
	X <- as.matrix(dat$X[[1]])
	Z <- as.matrix(dat$X[[2]])
	if(length(w) != 0) W <- dat$list1$w
	unitid1 <- dat$list1$unitid
	timeid1 <- dat$list1$timeid
	XX <- getcols(X,is.na(fmatch(colnames(X),colnames(Z)))) # endogenous
	ZZ <- getcols(Z,is.na(fmatch(colnames(Z),colnames(X)))) # instr
	XZ <- getcols(X,!is.na(fmatch(colnames(X),colnames(Z)))) # exogenous
	L <- ncol(XX) # number of endo.

	# estimate the model
	res <- list()
	if(!liml) {
		# 2SLS
		if(length(w) == 0) {
			iv1 <- ivreg(y ~ X,~Z)
		} else {
			iv1 <- ivreg(y ~ X,~Z,weights=W)
		}
		res <- get.ses(iv1,se,timeid=timeid1,unitid=unitid1,lag=lag)
		if(length(unitid1) > 0) {
			res$Clusters <- length(unique(unitid1))
		}
	} else {
		if(length(w) != 0) stop("error in ivreg0 -- liml doesn't work with weights yet")
		# LIML
		Z1 <- cbind(1,Z)
		X1 <- cbind(1,X)
		XZ1 <- cbind(1,XZ)
		N <- nrow(Z1)
		R <- cbind(y,XX)
		M  <- diag(N) - Z1 %*% solve((t(Z1) %*% Z1)) %*% t(Z1)	
		Mi <- diag(N) - XZ1 %*% solve((t(XZ1) %*% XZ1)) %*% t(XZ1)
		S <- t(R) %*% Mi %*% R %*% solve(t(R) %*% M %*% R)
		lambda <- min(eigen(S)$values)
		res$N <- N
		bread <- solve(t(X1) %*% (diag(N) - lambda * M) %*% X1)
		res$theta <- as.vector(bread %*% t(X1) %*% (diag(N) - lambda * M) %*% y)
	
		eps <- y - X1 %*% res$theta
		v <- as.vector(eps) * ((diag(N) - lambda * M) %*% X1)
		if(se == "robust") {
			meat <- var(v)
			res$V <- N * bread %*% meat %*% bread
		} else {
			I <- length(unique(unitid1))
			v2 <- matrix(rep(NA,I*ncol(v)),nrow=I)
			for(j in 1:ncol(v)) {
				v2[,j] <- aggregate(v[,j],by=list(unitid1),FUN=sum)[[2]]
			}
			meat <- var(v2)
			res$V <- I * bread %*% meat %*% bread
			res$Clusters <- I
		}
		res$se <- diag(res$V)^.5
	}
	res$ivs <- c("(Intercept)",colnames(X)) # rename for consistency
	
	# weak instrument check:
	for(l in 1:L) {
		if(ncol(XZ) > 0) {
			if(length(w) > 0) {
				lm1 <- lm0(XX[,l] ~ ZZ + XZ,se="classic",w=W)
			} else {
				lm1 <- lm0(XX[,l] ~ ZZ + XZ,se="classic")
			}
		} else {
			if(length(w) > 0) {
				lm1 <- lm0(XX[,l] ~ ZZ,se="classic",w=W)
			} else {
				print0("HERE!!")
				lm1 <- lm0(XX[,l] ~ ZZ,se="classic")
			}
		}
		res[[paste("FirstStageF_",l,sep="")]] <- F.test(function(x) x[fmatch(paste("ZZ",colnames(ZZ),sep=""),lm1$ivs)],lm1$N,lm1$coef,lm1$V)$F
	}
	res$r.squared <- NULL
	res
}

# compute variance-covariance matrix for func1(betahat), where func1 is vector-valued
delta.method <- function(func1,beta,V,...)
{
	c_beta <- jacobian(func1,beta,method="simple",...)
	Vc <- c_beta %*% V %*% t(c_beta)
	list(est=func1(beta,...),se=sqrt(diag(Vc)),V=Vc)
}

# test of a vector of nonlinear restrictions
F.test <- function(func1,N,beta,V,...)
{
	dm <- delta.method(func1,beta,V,...)
	cbeta <- func1(beta,...)
	J <- length(cbeta)
	K <- nrow(V)
	F <- t(cbeta) %*% solve(dm$V) %*% cbeta / J
	pval <- 1 - pf(F,J,N-K)
	list(F=F,df1=J,df2=N-K,pval=pval)
}

dig <- function(x,k=3)
{
	y <- format(round(x,k),nsmall=k)
	for(i in 1:length(y)) {
		if(is.na(x[i])) y[i] <- ""
	}
	y
}

dig1 <- function(x)
{
	dig(x,1)
}

dig3 <- function(x)
{
	dig(x,3)
}

as.coef <- function(coef,se,k=3,p.val=logical(0))
{
	if(length(p.val) == 0) {
		if(is.matrix(coef)) {
			res <- coef
			for(i in 1:ncol(coef)) res[,i] <- paste(dig(coef[,i],k),stars(coef[,i],se[,i]),sep="")
			res
		} else {
			paste(dig(coef,k),stars(coef,se),sep="")
		}
	} else {
		if(is.matrix(coef)) {
			res <- coef
			for(i in 1:ncol(coef)) res[,i] <- paste(dig(coef[,i],k),stars.pval(coef[,i],p.val[,i]),sep="")
			res
		} else {
			paste(dig(coef,k),stars.pval(p.val),sep="")
		}
	}
}

as.se <- function(se,k=3)
{
	if(is.matrix(se)) {
		res <- se
		for(i in 1:ncol(se)) res[,i] <- str_replace_all(paste("(",dig(se[,i],k),")",sep=""),"\\(\\)","")
		res
	} else {
		str_replace_all(paste("(",dig(se,k),")",sep=""),"\\(\\)","")
	}
}

pval <- function(est,se)
{
	2*pnorm(-abs(est/se))
}

as.pval <- function(coef,se,k=3)
{
	paste(dig(pval(coef,se),k),stars(coef,se),sep="")
}

stars.scalar <- function(coef,se)
{
	if(is.na(coef)) {
		""
	} else if(is.na(se)) {
		"?"
	} else {
		pval1 <- pval(coef,se)
		if(pval1 <= 0.001) {
			"***"
		} else if(pval1 <= 0.01) {
			"**"
		} else if(pval1 <= 0.05) {
			"*"
		} else if(pval1 <= 0.1) {
			"+"
		} else {
			""
		}
	}
}

stars <- function(coef,se)
{
	if(is.matrix(coef))
	{
		m <- nrow(coef)
		n <- ncol(coef)
		res <- coef
		for(i in 1:m) for(j in 1:n) res[i,j] <- stars.scalar(coef[i,j],se[i,j])
	} else {
		m <- length(coef)
		res <- coef
		for(i in 1:m) res[i] <- stars.scalar(coef[i],se[i])
	}
	res
}

stars.pval <- function(pval1)
{
	n <- length(pval1)
	ret <- rep(NA,n)
	for(i in 1:n)
	{
		if(pval1[i] <= 0.001) {
			ret[i] <- "***"
		} else if(pval1[i] <= 0.01) {
			ret[i] <- "**"
		} else if(pval1[i] <= 0.05) {
			ret[i] <- "*"
		} else if(pval1[i] <= 0.1) {
			ret[i] <- "+"
		} else {
			ret[i] <- ""
		}
	}
	ret
}

pval.as.pval <- function(pval,k=3)
{
	paste(dig(pval,k),stars.pval(pval),sep="")
}

latex_doc_start <- function(file1,packages=logical(0))
{
	cat("\\documentclass[11pt]{article}",file=file1,sep="\n")
	s <- "\\usepackage{float,amsmath,amssymb,amsthm,fullpage,setspace,natbib,enumerate,url,multirow,lscape"
	if(length(packages) > 0) for(i in 1:length(packages)) s <- paste(s,",",packages[i],sep="")
	s <- paste(s,"}",sep="")
	cat(s,file=file1,sep="\n",append=TRUE)
	#cat("\\usepackage[dvips]{graphicx,xcolor}",file=file1,sep="\n",append=TRUE)
	cat("\\onehalfspacing",file=file1,sep="\n",append=TRUE)
	cat("\\begin{document}",file=file1,sep="\n\n",append=TRUE)
}

latex_doc_end <- function(file1)
{
	cat("\\end{document}",file=file1,sep="\n",append=TRUE)
}

latex_table_start <- function(file1,k)
{
	cat("\\begin{table}[htp]\\centering\\scriptsize",file=file1,sep="\n",append=TRUE)
	cat("\\begin{singlespace}",file=file1,sep="\n",append=TRUE)
	cat("\\begin{tabular}{ l ",paste(rep("c",k),collapse=" ")," } \\hline \\hline",file=file1,sep="\n",append=TRUE)
}

latex_table_end <- function(file1,caption="",label="")
{
	cat("\\hline \\hline",file=file1,sep="\n",append=TRUE)
	cat("\\end{tabular}",file=file1,sep="\n",append=TRUE)
	cat("\\end{singlespace}",file=file1,sep="\n",append=TRUE)
	if(caption != "") cat(paste0(c("\\caption{\\footnotesize{",caption,"}}")),file=file1,sep="\n",append=TRUE)
	if(label != "") cat(paste0(c("\\label{",label,"}")),file=file1,sep="\n",append=TRUE)
	cat("\\end{table}",file=file1,sep="\n",append=TRUE)
}

latex_add_line <- function(file1,s)
{
	cat(s,file=file1,sep="\n",append=TRUE)
}

latex_table_entry <- function(file1,name,coef,bold=FALSE,italic=FALSE)
{
	if(bold)
	{
		name <- paste("\\textbf{",name,"}",sep="")
		coef <- paste("\\textbf{",coef,"}",sep="")
	}
	if(italic)
	{
		name <- paste("\\textit{",name,"}",sep="")
		coef <- paste("\\textit{",coef,"}",sep="")
	}
	s <- name
	n <- length(coef)
	for(i in 1:n) if(is.na(coef[i])) { coef[i] <- "" }
	if(n > 1) for(i in 1:(n-1)) { s <- paste(s," & ",coef[i],sep="") }
	s <- paste(s," & ",coef[n]," \\\\ \n",sep="")
	s
	cat(s,file=file1,sep="\n",append=TRUE)
}

latex_table_entries <- function(file1,names,coefs,bold=FALSE,italic=FALSE)
{
	m <- length(names)
	for(j in 1:m) latex_table_entry(file1,names[j],coefs[j,],bold)
}

# main data file
data1 <- read.dta(paste(dir1,"cand_place_rep.dta",sep="")) # cces and other data
n <- nrow(data1) # number of respondents

# race type
canddem <- data1$canddem # 1 if candidate 1 is a dem.
candrep <- data1$candrep # 1 if candidate 2 is a rep.
samp1 <- (canddem & candrep) # sample consists of races where a dem. cand. faced an rep. cand.

# resp. stuff
w1 <- data1$w1 # resp. weights
statecd <- data1$statecd # state/cong. dist.
dem <- data1$dem # democratic voter
rep <- data1$rep # republican voter
housevote <- data1$housevote # vote in house election
lowknow <- data1$lowknow # low knowledge dummy
highknow <- data1$highknow # high knowledge dummy
selfplacement <- data1$selfplacement # self placement, untransformed
hou_dem_place0 <- data1$hou_dem_place0 # resp. placement of dem. cand., untransformed
hou_rep_place0 <- data1$hou_rep_place0 # resp. placement of rep. cand., untransformed

# cand. stuff
hou_dem_inc <- data1$hou_dem_inc # dem. cand. is an inc.
hou_rep_inc <- data1$hou_rep_inc # rep. cand. is an inc.
hou_dem_chq <- data1$hou_dem_chq # dem. cand., chal. quality
hou_rep_chq <- data1$hou_rep_chq # rep. cand., chal. quality
stunem <- data1$stunem # state unemployment
party_gov <- data1$party_gov # party of governor
stunem_party_gov <- stunem * party_gov

# ideal point estimates
voter_pos <- data1$voter_pos # estimated position, cces resp.
hou_med_voter <- data1$hou_med_voter # median voter in the dist.
hou_dem_pos_1 <- data1$hou_dem_pos_1 # estimated position, dem. house cand.
hou_rep_pos_1 <- data1$hou_rep_pos_1 # estimated position, rep. house cand.

# npat estimates
hou_dem_pos0_npat <- data1$hou_dem_pos0_npat # npat dem. house cand. position, untransformed
hou_rep_pos0_npat <- data1$hou_rep_pos0_npat # npat rep. house cand. position, untransformed
hou_dem_pos0_old_npat <- data1$hou_dem_pos0_old_npat # npat dem. house cand. position, prev. cycle, untransformed
hou_rep_pos0_old_npat <- data1$hou_rep_pos0_old_npat # npat rep. house cand. position, prev. cycle, untransformed
cue_sen_dem_pos0_npat <- data1$cue_sen_dem_pos0_npat # npat for most recent dem. sen. cand., untransformed
cue_sen_rep_pos0_npat <- data1$cue_sen_rep_pos0_npat # npat for most recent rep. sen. cand., untransformed

# bonica estimates
hou_dem_pos0_cf <- data1$hou_dem_pos0_cf # bonica dem. house cand. position, untransformed
hou_rep_pos0_cf <- data1$hou_rep_pos0_cf # bonica rep. house cand. position, untransformed
cue_sen_dem_pos0_cf <- data1$cue_sen_dem_pos0_cf # bonica for most recent dem. sen. cand., untrasformed
cue_sen_rep_pos0_cf <- data1$cue_sen_rep_pos0_cf # bonica for most recent rep. sen. cand., untrasformed
cue_gov_dem_pos0_cf <- data1$cue_gov_dem_pos0_cf # bonica for most recent dem. gov. cand., untrasformed
cue_gov_rep_pos0_cf <- data1$cue_gov_rep_pos0_cf # bonica for most recent rep. gov. cand., untrasformed

# linear map candidate placements
place_map <- summary(lm(voter_pos ~ selfplacement))$coef
hou_dem_place <- place_map[1] + place_map[2] * hou_dem_place0
hou_rep_place <- place_map[1] + place_map[2] * hou_rep_place0 

# linear map npat
statecds <- sort(unique(statecd))
pos0_npat <- c(mean0(hou_dem_pos0_npat,by=statecd,bys=statecds),mean0(hou_rep_pos0_npat,by=statecd,bys=statecds)) # npat, un-normalized
pos_1     <- c(mean0(hou_dem_pos_1    ,by=statecd,bys=statecds),mean0(hou_rep_pos_1    ,by=statecd,bys=statecds)) # common space
npat_map <- lm0(pos_1 ~ pos0_npat)$coef
hou_dem_pos_npat     <- npat_map[1] + npat_map[2] * hou_dem_pos0_npat
hou_rep_pos_npat     <- npat_map[1] + npat_map[2] * hou_rep_pos0_npat
hou_dem_pos_old_npat <- npat_map[1] + npat_map[2] * hou_dem_pos0_old_npat 
hou_rep_pos_old_npat <- npat_map[1] + npat_map[2] * hou_rep_pos0_old_npat
cue_sen_dem_pos_npat <- npat_map[1] + npat_map[2] * cue_sen_dem_pos0_npat
cue_sen_rep_pos_npat <- npat_map[1] + npat_map[2] * cue_sen_rep_pos0_npat

# linear map bonica
pos0_cf   <- c(mean0(hou_dem_pos0_cf  ,by=statecd,bys=statecds),mean0(hou_rep_pos0_cf  ,by=statecd,bys=statecds)) # bonica, un-normalized
cf_map   <- lm0(pos_1 ~ pos0_cf)$coef
hou_dem_pos_cf     <- cf_map[1] + cf_map[2] * hou_dem_pos0_cf
hou_rep_pos_cf     <- cf_map[1] + cf_map[2] * hou_rep_pos0_cf
cue_sen_dem_pos_cf <- cf_map[1] + cf_map[2] * cue_sen_dem_pos0_cf
cue_sen_rep_pos_cf <- cf_map[1] + cf_map[2] * cue_sen_rep_pos0_cf
cue_gov_dem_pos_cf <- cf_map[1] + cf_map[2] * cue_gov_dem_pos0_cf
cue_gov_rep_pos_cf <- cf_map[1] + cf_map[2] * cue_gov_rep_pos0_cf

# roll call estimate, or npat is missing
hou_dem_pos_2 <- hou_dem_pos_1
hou_dem_pos_2[is.na(hou_dem_pos_2)] <- hou_dem_pos_npat[is.na(hou_dem_pos_2)]
hou_rep_pos_2 <- hou_rep_pos_1
hou_rep_pos_2[is.na(hou_rep_pos_2)] <- hou_rep_pos_npat[is.na(hou_rep_pos_2)]

# roll call estimate, or npat is missing, or bonica if still missing
hou_dem_pos_3 <- hou_dem_pos_2
hou_dem_pos_3[is.na(hou_dem_pos_3)] <- hou_dem_pos_cf[is.na(hou_dem_pos_3)]
hou_rep_pos_3 <- hou_rep_pos_2
hou_rep_pos_3[is.na(hou_rep_pos_3)] <- hou_rep_pos_cf[is.na(hou_rep_pos_3)]

# most recent senate candidate, npat
cue_dem_sen_2 <- cue_sen_dem_pos_npat
cue_rep_sen_2 <- cue_sen_rep_pos_npat

# npat, or bonica if missing
cue_dem_sen_3 <- cue_dem_sen_2
cue_dem_sen_3[is.na(cue_dem_sen_3)] <- cue_sen_dem_pos_cf[is.na(cue_dem_sen_3)]
cue_rep_sen_3 <- cue_rep_sen_2
cue_rep_sen_3[is.na(cue_rep_sen_3)] <- cue_sen_rep_pos_cf[is.na(cue_rep_sen_3)]

# cue based on npat for senate, bonica for gov
hou_dem_cue_2 <- rowMeans(cbind(cue_dem_sen_2,cue_gov_dem_pos_cf))
hou_rep_cue_2 <- rowMeans(cbind(cue_rep_sen_2,cue_gov_rep_pos_cf))

# cue based on npat for senate, or bonica if missing, and bonica for gov
hou_dem_cue_3 <- rowMeans(cbind(cue_dem_sen_3,cue_gov_dem_pos_cf))
hou_rep_cue_3 <- rowMeans(cbind(cue_rep_sen_3,cue_gov_rep_pos_cf))

# signed differences (used for plots testing assimilation/contrast and cheerleading)
hou_dem_voter_pos_diff_3    <- hou_dem_pos_3 - voter_pos
hou_rep_voter_pos_diff_3    <- hou_rep_pos_3 - voter_pos
hou_dem_medvoter_pos_diff_3 <- hou_dem_pos_3 - hou_med_voter
hou_rep_medvoter_pos_diff_3 <- hou_rep_pos_3 - hou_med_voter
hou_dem_voter_place_diff <- hou_dem_place - voter_pos
hou_rep_voter_place_diff <- hou_rep_place - voter_pos
hou_dem_medvoter_place_diff <- hou_dem_place - hou_med_voter
hou_rep_medvoter_place_diff <- hou_rep_place - hou_med_voter

# main analysis uses roll call, or npat if missing, or bonica if still missing
hou_dem_pos <- hou_dem_pos_3
hou_rep_pos <- hou_rep_pos_3
hou_dem_cue <- hou_dem_cue_3
hou_rep_cue <- hou_rep_cue_3
hou_dem_voter_pos_diff <- hou_dem_voter_pos_diff_3
hou_rep_voter_pos_diff <- hou_rep_voter_pos_diff_3
hou_dem_medvoter_pos_diff <- hou_dem_medvoter_pos_diff_3
hou_rep_medvoter_pos_diff <- hou_rep_medvoter_pos_diff_3

# bridge vote data
data2 <- read.csv(paste(dir1,"YB.csv",sep=""))
YB <- data2[,1:10]
party <- data2$party
chamber <- data2$chamber
chamberparty <- paste(chamber,party,sep="")
bridges <- str_replace_all(str_replace_all(str_replace_all(str_replace_all(colnames(YB),"\\."," ")," c ","\\(c\\)")," l ","\\(l\\)")," n a ","\\(n/a\\)")

# figure 1 #
pdf(file=paste(dir1,"fig1.pdf",sep=""),width=12,height=6.2)
plot(0:1,type='n',xlim=c(-3.4,3.4),ylim=c(0,1.2),xlab="Position",ylab="")
points(density0(voter_pos),type='l',lty=2)
points(density0(subset(voter_pos,dem==1)),type='l',lty=2,col='blue')
points(density0(subset(voter_pos,rep==1)),type='l',lty=2,col='red')
points(density0(hou_dem_pos_3,bw=.3),type='l',col='blue')
points(density0(hou_rep_pos_3,bw=.3),type='l',col='red')
points(density0(hou_dem_place,bw=.3),type='l',col='blue',lty=3)
points(density0(hou_rep_place,bw=.3),type='l',col='red',lty=3)
legend(c(1.5,3.4),c(.525,1),lty=c(2,2,2,1,1,3,3),legend=c("Voters","Dem. Voters","Rep. Voters","Dem. Cand.","Rep. Cand.","Placements of Dem. Cand.","Placements of Rep. Cand."),col=c('black','blue','red','blue','red','blue','red'))
dev.off()

# table 1 #
labels1 <- rep(NA,16)
vec1 <- matrix(rep(NA,16),ncol=1)
labels1[1] <- "Average Democratic candidate"
vec1[1,] <- dig(median0(hou_dem_pos,subset=samp1))
labels1[2] <- "Average Democratic respondent"
vec1[2,] <- dig(median0(voter_pos,subset=samp1&dem==1))
labels1[3] <- "Average placement of Democratic candidate"
vec1[3,] <- dig(mean0(hou_dem_place,subset=samp1))
labels1[4] <- "Average respondent"
vec1[4,] <- dig(median0(voter_pos,subset=samp1))
labels1[5] <- "Average placement of Republican candidate"
vec1[5,] <- dig(mean0(hou_rep_place,subset=samp1))
labels1[6] <- "Average Republican respondent"
vec1[6,] <- dig(median0(voter_pos,subset=samp1&rep==1))
labels1[7] <- "Average Republican candidate"
vec1[7,] <- dig(median0(hou_rep_pos,subset=samp1))
labels1[8] <- "Distance between average voter and Democratic candidate"
vec1[8,] <- dig(median0(voter_pos,subset=samp1)-median0(hou_dem_pos,subset=samp1))
labels1[9] <- "Distance between average voter and Republican candidate"
vec1[9,] <- dig(median0(hou_rep_pos,subset=samp1)-median0(voter_pos,subset=samp1))
labels1[10] <- "Distance between average voter and placement of Democratic candidate"
vec1[10,] <- dig(mean0(voter_pos,subset=samp1)-mean0(hou_dem_place,subset=samp1))
labels1[11] <- "Distance between average voter and placement of Republican candidate"
vec1[11,] <- dig(mean0(hou_rep_place,subset=samp1)-mean0(voter_pos,subset=samp1))
labels1[12] <- "Actual distance between average Democratic and Republican House candidate"
vec1[12,] <- dig(median0(hou_rep_pos,subset=samp1)-median0(hou_dem_pos,subset=samp1))
labels1[13] <- "Distance between average Democratic and Republican identifier"
vec1[13,] <- dig(median0(voter_pos,subset=samp1&rep==1)-median0(voter_pos,subset=samp1&dem==1))
labels1[14] <- "Average perceived distance between Democratic and Republican House candidate"
vec1[14,] <- dig(mean0(hou_rep_place,subset=samp1)-mean0(hou_dem_place,subset=samp1))
labels1[15] <- "\\hspace{.3cm}\\textit{High Knowledge Respondents}"
vec1[15,] <- dig(mean0(hou_rep_place,subset=samp1&highknow)-mean0(hou_dem_place,subset=samp1&highknow))
labels1[16] <- "\\hspace{.3cm}\\textit{Low Knowledge Respondents}"
vec1[16,] <- dig(mean0(hou_rep_place,subset=samp1&lowknow)-mean0(hou_dem_place,subset=samp1&lowknow))

# figure 2 #
pdf(file=paste(dir1,"fig2.pdf",sep=""),width=9,height=9)
par(mfrow=c(2,2))

plot(0:1,type='n',xlim=c(-4,4),ylim=c(-4,4),main="Dem. Voters Place Dem. Candidates",xlab="Cand. Pos. - Voter Pos.",ylab="Cand. Place. - Voter Pos.")
polygon(c(-5,-5,0),c(5,-5,0),col='lightgrey',border=NA)
polygon(c(5,5,0),c(5,-5,0),col='lightgrey',border=NA)
text(subset(hou_dem_voter_pos_diff,housevote==0),subset(hou_dem_voter_place_diff,housevote==0),'.',col='blue')
points(-4:4,-4:4,type='l',lty=2)
abline(lm0(hou_dem_voter_place_diff ~ hou_dem_voter_pos_diff,subset=housevote==0))

plot(0:1,type='n',xlim=c(-4,4),ylim=c(-4,4),main="Rep. Voters Place Dem. Candidates",xlab="Cand. Pos. - Voter Pos.",ylab="Cand. Place. - Voter Pos.")
polygon(c(-5,-5,0),c(5,-5,0),col='lightgrey',border=NA)
polygon(c(5,5,0),c(5,-5,0),col='lightgrey',border=NA)
text(subset(hou_dem_voter_pos_diff,housevote==1),subset(hou_dem_voter_place_diff,housevote==1),'.',col='blue')
points(-4:4,-4:4,type='l',lty=2)
abline(lm0(hou_dem_voter_place_diff ~ hou_dem_voter_pos_diff,subset=housevote==1))

plot(0:1,type='n',xlim=c(-4,4),ylim=c(-4,4),main="Dem. Voters Place Rep. Candidates",xlab="Cand. Pos. - Voter Pos.",ylab="Cand. Place. - Voter Pos.")
polygon(c(-5,-5,0),c(5,-5,0),col='lightgrey',border=NA)
polygon(c(5,5,0),c(5,-5,0),col='lightgrey',border=NA)
text(subset(hou_rep_voter_pos_diff,housevote==0),subset(hou_rep_voter_place_diff,housevote==0),'.',col='red')
points(-4:4,-4:4,type='l',lty=2)
abline(lm0(hou_rep_voter_place_diff ~ hou_rep_voter_pos_diff,subset=housevote==0))

plot(0:1,type='n',xlim=c(-4,4),ylim=c(-4,4),main="Rep. Voters Place Rep. Candidates",xlab="Cand. Pos. - Voter Pos.",ylab="Cand. Place. - Voter Pos.")
polygon(c(-5,-5,0),c(5,-5,0),col='lightgrey',border=NA)
polygon(c(5,5,0),c(5,-5,0),col='lightgrey',border=NA)
text(subset(hou_rep_voter_pos_diff,housevote==1),subset(hou_rep_voter_place_diff,housevote==1),'.',col='red')
points(-4:4,-4:4,type='l',lty=2)
abline(lm0(hou_rep_voter_place_diff ~ hou_rep_voter_pos_diff,subset=housevote==1))
dev.off()

# table 2 #
lms <- list()
lm1 <- lm0(hou_dem_voter_place_diff ~ hou_dem_voter_pos_diff,subset=housevote==1,se="cluster",unitid=statecd,nointercept=TRUE)
lm2 <- lm0(hou_dem_voter_place_diff ~ hou_dem_voter_pos_diff,subset=housevote==0,se="cluster",unitid=statecd,nointercept=TRUE)
lm3 <- lm0(hou_rep_voter_place_diff ~ hou_rep_voter_pos_diff,subset=housevote==0,se="cluster",unitid=statecd,nointercept=TRUE)
lm4 <- lm0(hou_rep_voter_place_diff ~ hou_rep_voter_pos_diff,subset=housevote==1,se="cluster",unitid=statecd,nointercept=TRUE)

mat1 <- matrix(rep(NA,4*8),nrow=8)
mat1[1,1] <- as.coef(lm1$coef[1],lm1$se[1])
mat1[2,1] <- as.se(lm1$se[1])
mat1[3,2] <- as.coef(lm2$coef[1],lm2$se[1])
mat1[4,2] <- as.se(lm2$se[1])
mat1[1,3] <- as.coef(lm3$coef[1],lm3$se[1])
mat1[2,3] <- as.se(lm3$se[1])
mat1[3,4] <- as.coef(lm4$coef[1],lm4$se[1])
mat1[4,4] <- as.se(lm4$se[1])
mat1[5,1] <- lm1$N
mat1[5,2] <- lm2$N
mat1[5,3] <- lm3$N
mat1[5,4] <- lm4$N
mat1[6,1] <- lm1$Clusters
mat1[6,2] <- lm2$Clusters
mat1[6,3] <- lm3$Clusters
mat1[6,4] <- lm4$Clusters
mat1[7,1] <- pval.as.pval(pnorm(-abs((lm1$coef[1]-1)/lm1$se[1])))
mat1[8,2] <- pval.as.pval(pnorm(-abs((lm2$coef[1]-1)/lm2$se[1])))
mat1[7,3] <- pval.as.pval(pnorm(-abs((lm3$coef[1]-1)/lm3$se[1])))
mat1[8,4] <- pval.as.pval(pnorm(-abs((lm4$coef[1]-1)/lm4$se[1])))

# figure 3 #
pdf(file=paste(dir1,"fig3.pdf",sep=""),width=9,height=9)
par(mfrow=c(2,2))

plot(0:1,type='n',xlim=c(-4,4),ylim=c(-4,4),main="Dem. Voters Place Dem. Candidates",xlab="Cand. Pos. - Med. Voter Pos.",ylab="Cand. Place. - Med. Voter Pos.")
polygon(c(-5,-5,0),c(5,-5,0),col='lightgrey',border=NA)
polygon(c(5,5,0),c(5,-5,0),col='lightgrey',border=NA)
text(subset(hou_dem_medvoter_pos_diff,housevote==0),subset(hou_dem_medvoter_place_diff,housevote==0),'.',col='blue')
points(-4:4,-4:4,type='l',lty=2)
abline(lm0(hou_dem_medvoter_place_diff ~ hou_dem_medvoter_pos_diff,subset=housevote==0))

plot(0:1,type='n',xlim=c(-4,4),ylim=c(-4,4),main="Rep. Voters Place Dem. Candidates",xlab="Cand. Pos. - Med. Voter Pos.",ylab="Cand. Place. - Med. Voter Pos.")
polygon(c(-5,-5,0),c(5,-5,0),col='lightgrey',border=NA)
polygon(c(5,5,0),c(5,-5,0),col='lightgrey',border=NA)
text(subset(hou_dem_medvoter_pos_diff,housevote==1),subset(hou_dem_medvoter_place_diff,housevote==1),'.',col='blue')
points(-4:4,-4:4,type='l',lty=2)
abline(lm0(hou_dem_medvoter_place_diff ~ hou_dem_medvoter_pos_diff,subset=housevote==1))

plot(0:1,type='n',xlim=c(-4,4),ylim=c(-4,4),main="Dem. Voters Place Rep. Candidates",xlab="Cand. Pos. - Med. Voter Pos.",ylab="Cand. Place. - Med. Voter Pos.")
polygon(c(-5,-5,0),c(5,-5,0),col='lightgrey',border=NA)
polygon(c(5,5,0),c(5,-5,0),col='lightgrey',border=NA)
text(subset(hou_rep_medvoter_pos_diff,housevote==0),subset(hou_rep_medvoter_place_diff,housevote==0),'.',col='red')
points(-4:4,-4:4,type='l',lty=2)
abline(lm0(hou_rep_medvoter_place_diff ~ hou_rep_medvoter_pos_diff,subset=housevote==0))

plot(0:1,type='n',xlim=c(-4,4),ylim=c(-4,4),main="Rep. Voters Place Rep. Candidates",xlab="Cand. Pos. - Med. Voter Pos.",ylab="Cand. Place. - Med. Voter Pos.")
polygon(c(-5,-5,0),c(5,-5,0),col='lightgrey',border=NA)
polygon(c(5,5,0),c(5,-5,0),col='lightgrey',border=NA)
text(subset(hou_rep_medvoter_pos_diff,housevote==1),subset(hou_rep_medvoter_place_diff,housevote==1),'.',col='red')
points(-4:4,-4:4,type='l',lty=2)
abline(lm0(hou_rep_medvoter_place_diff ~ hou_rep_medvoter_pos_diff,subset=housevote==1))
dev.off()

# table 3 #
lm1 <- lm0(hou_dem_medvoter_place_diff ~ hou_dem_medvoter_pos_diff,subset=housevote==1,se="cluster",unit=statecd,nointercept=TRUE)
lm2 <- lm0(hou_dem_medvoter_place_diff ~ hou_dem_medvoter_pos_diff,subset=housevote==0,se="cluster",unit=statecd,nointercept=TRUE)
lm3 <- lm0(hou_rep_medvoter_place_diff ~ hou_rep_medvoter_pos_diff,subset=housevote==0,se="cluster",unit=statecd,nointercept=TRUE)
lm4 <- lm0(hou_rep_medvoter_place_diff ~ hou_rep_medvoter_pos_diff,subset=housevote==1,se="cluster",unit=statecd,nointercept=TRUE)

mat2 <- matrix(rep(NA,4*8),nrow=8)
mat2[1,1] <- as.coef(lm1$coef[1],lm1$se[1])
mat2[2,1] <- as.se(lm1$se[1])
mat2[3,2] <- as.coef(lm2$coef[1],lm2$se[1])
mat2[4,2] <- as.se(lm2$se[1])
mat2[1,3] <- as.coef(lm3$coef[1],lm3$se[1])
mat2[2,3] <- as.se(lm3$se[1])
mat2[3,4] <- as.coef(lm4$coef[1],lm4$se[1])
mat2[4,4] <- as.se(lm4$se[1])
mat2[5,1] <- lm1$N
mat2[5,2] <- lm2$N
mat2[5,3] <- lm3$N
mat2[5,4] <- lm4$N
mat2[6,1] <- lm1$Clusters
mat2[6,2] <- lm2$Clusters
mat2[6,3] <- lm3$Clusters
mat2[6,4] <- lm4$Clusters
mat2[7,1] <- pval.as.pval(pnorm(-abs((lm1$coef[1]-1)/lm1$se[1])))
mat2[8,2] <- pval.as.pval(pnorm(-abs((lm2$coef[1]-1)/lm2$se[1])))
mat2[7,3] <- pval.as.pval(pnorm(-abs((lm3$coef[1]-1)/lm3$se[1])))
mat2[8,4] <- pval.as.pval(pnorm(-abs((lm4$coef[1]-1)/lm4$se[1])))

# table 4 #
lms <- list()
lms[[1]] <- lm0(hou_dem_place ~ hou_dem_pos + hou_dem_cue + hou_dem_inc,se="cluster",unitid=statecd,subset=samp1)
lms[[2]] <- lm0(hou_dem_place ~ hou_dem_pos + hou_dem_cue + hou_dem_inc,se="cluster",unitid=statecd,subset=samp1&highknow)
lms[[3]] <- lm0(hou_dem_place ~ hou_dem_pos + hou_dem_cue + hou_dem_inc,se="cluster",unitid=statecd,subset=samp1&lowknow)
lms[[4]] <- lm0(hou_rep_place ~ hou_rep_pos + hou_rep_cue + hou_rep_inc,se="cluster",unitid=statecd,subset=samp1)
lms[[5]] <- lm0(hou_rep_place ~ hou_rep_pos + hou_rep_cue + hou_rep_inc,se="cluster",unitid=statecd,subset=samp1&highknow)
lms[[6]] <- lm0(hou_rep_place ~ hou_rep_pos + hou_rep_cue + hou_rep_inc,se="cluster",unitid=statecd,subset=samp1&lowknow)

mat3 <- matrix(rep(NA,12*6),nrow=12)
for(i in 1:6)
{
	mat3[c(1,3,5,7),i] <- as.coef(lms[[i]]$coef,lms[[i]]$se)
	mat3[c(2,4,6,8),i] <- as.se(lms[[i]]$se)
	mat3[9,i] <- lms[[i]]$N
	mat3[10,i] <- lms[[i]]$Clusters
	mat3[11,i] <- dig3(lms[[i]]$r.squared)
	mat3[12,i] <- as.se(lms[[i]]$sigmasqse)
}

# IV GMM functions #

ivgmm <- function(theta,dat)
{
	n <- nrow(dat)
	k <- length(theta) - 4
	l <- ncol(dat) - 5 - k
	y <- dat[1:n,1]
	X <- dat[1:n,2]
	M <- dat[1:n,3]
	S <- dat[1:n,4:(4+k-1)]
	Z <- dat[1:n,(4+k):(4+k+l-1)]
	C <- dat[1:n,4+k+l]
	W <- dat[1:n,4+k+l+1]
	alpha <- theta[1]
	gamma <- theta[2]
	delta <- theta[3]
	tau <- theta[4]
	lambda <- theta[5:(5+k-1)]
	a <- (1 - gamma) * tau
	b <- 1 - tau
	c <- tau * gamma * lambda
	d <- delta - tau - delta * alpha + tau * gamma
	e <- tau - delta
	f <- (delta * alpha - tau * gamma) * lambda
	temp <- y - a * X - b * M - S %*% c - d * C * X - e * C * M
	for(i in 1:k) {
		temp <- temp - f[i] * C * S[1:n,i]
	}
	temp <- temp * W
	v <- matrix(rep(0,n*l),nrow=n)
	for(i in 1:l) {
		v[1:n,i] <- Z[1:n,i] * temp
	}
	v
}

ivgmm2 <- function(theta,dat)
{
	v <- ivgmm(theta,dat)
	vavg <- colSums(v) / nrow(v)
	vavg
}

ivgmm_run <- function(y,X,M,S,C,Z,W,opt=1,subset=logical(0),cluster=logical(0))
{
	if(length(subset) == 0) {
		miss <- is.na(y) | is.na(X) | is.na(M) | is.na(W) | !apply(S, 1, function(x)!any(is.na(x))) | is.na(C) | !apply(Z, 1, function(x)!any(is.na(x)))
	} else if(length(subset) == length(y)) {
		miss <- is.na(y) | is.na(X) | is.na(M) | is.na(W) | !apply(S, 1, function(x)!any(is.na(x))) | is.na(C) | !apply(Z, 1, function(x)!any(is.na(x))) | !subset
	} else {
		stop("Incompadible Dimensions")
	}
	y2 <- y[!miss]
	X2 <- X[!miss]
	M2 <- M[!miss]
	S2 <- S[!miss,]
	C2 <- C[!miss]
	Z2 <- Z[!miss,]
	W2 <- W[!miss]
	n <- length(y2)
	if(length(cluster) == length(y)) {
		cluster2 <- cluster[!miss]
	} else {
		cluster2 <- logical(0)
	}

	dropZ <- rep(FALSE,ncol(Z2))
	for(i in 1:ncol(Z2)) {
		if(var(Z2[,i]) == 0) dropZ[i] <- TRUE
	}
	Z2 <- Z2[,!dropZ]

	# first stage
	lm1 <- lm(C2 ~ Z2,weights=W2)
	
	dat <- cbind(y2,X2,M2,S2,Z2,C2,W2)
	if(opt < 5) {
		theta0 <- c(1,1,1,1,0,0,0,0)
		names(theta0) <- c("alpha (assim)","gamma (contr)","delta (cheer)","tau (anti-cheer)","lambda1 (const)","lambda2 (pos)","lambda3 (cue)","lambda4 (inc)")
	} else if(opt == 5) {
		theta0 <- c(1,1,1,1,0,0,0)
		names(theta0) <- c("alpha (assim)","gamma (contr)","delta (cheer)","tau (anti-cheer)","lambda1 (const)","lambda2 (pos)","lambda3 (cue)")
	} else if(opt == 6) {
		theta0 <- c(1,1,1,1,0,0,0,0)
		names(theta0) <- c("alpha (assim)","gamma (contr)","delta (cheer)","tau (anti-cheer)","lambda1 (const)","lambda2 (pos)","lambda3 (cue)","lambda4 (dem)")
	} else {
		theta0 <- c(1,1,1,1,0,0,0,0,0)
		names(theta0) <- c("alpha (assim)","gamma (contr)","delta (cheer)","tau (anti-cheer)","lambda1 (const)","lambda2 (pos)","lambda3 (cue)","lambda4 (old)","lambda5 (inc)")
	}
	
	if(opt == 1 || opt == 5 || opt == 6 || opt == 7) {
		gmm1 <- gmm(ivgmm, x = dat, t0 = theta0,vcov="iid")
	} else if(opt == 2) {
		gmm1 <- gmm(ivgmm, x = dat, t0 = theta0,vcov="iid",eqConst = c(3,4))
	} else if(opt == 3) {
		gmm1 <- gmm(ivgmm, x = dat, t0 = theta0,vcov="iid",eqConst = c(1,2))
	} else {
		gmm1 <- gmm(ivgmm, x = dat, t0 = theta0,vcov="iid",eqConst = c(1,2,3,4))
	}	

	theta1 <- gmm1$coefficients
	if(opt == 2) {
		theta1 <- c(theta1[1:2],1,1,theta1[3:6])
	} else if(opt == 3) {
		theta1 <- c(1,1,theta1[1:6])
	} else if(opt == 4) {
		theta1 <- c(1,1,1,1,theta1[1:4])
	}

	G <- jacobian(ivgmm2,theta1,dat=dat)
	v <- ivgmm(theta1,dat=dat)
	resid <- v[,4]	
	
	if(length(cluster2) == 0) {
		omega <- var(v)
	} else if(length(cluster2) == n) {
		I <- length(unique(cluster2))
		v2 <- matrix(rep(NA,I*ncol(v)),nrow=I)
		for(j in 1:ncol(v)) {
			v2[,j] <- (I / n) * aggregate(v[,j],by=list(cluster2),FUN=sum)[[2]]
		}
		omega <- var(v2)
	}
	w <- gmm1$w

	C <- t(G) %*% w %*% G
	B <- t(G) %*% w %*% omega %*% t(w) %*% G

	if(length(cluster2) == 0) {
		V <- solve(C) %*% B %*% solve(C) / n
	} else {
		V <- solve(C) %*% B %*% solve(C) / I
	}
	thetase1 <- diag(V)^.5

	if(length(cluster2) == 0) {
		list(theta=theta1,thetase=thetase1,V=V,N=n,firststage=summary(lm1))
	} else {
		list(theta=theta1,thetase=thetase1,V=V,N=n,Clusters=I,firststage=summary(lm1))
	}
}

# table 5 #
instr <- cbind(hou_dem_chq,hou_rep_chq,stunem,stunem_party_gov)
instr_hou_voter_pos <- matrix(rep(NA,ncol(instr)*n),nrow=n)
instr_hou_med_voter <- matrix(rep(NA,ncol(instr)*n),nrow=n)
for(j in 1:ncol(instr))
{
	instr_hou_voter_pos[1:n,j] <- instr[1:n,j] * voter_pos
	instr_hou_med_voter[1:n,j] <- instr[1:n,j] * hou_med_voter
}
X <- voter_pos
M <- hou_med_voter
wald1234_1 <- function(x) { c(x[1] - 1,x[2] - 1,x[3] - 1,x[4] - 1) }
wald67 <- function(x) { x[6] + x[7] }
gmms <- list()

SD <- cbind(1,hou_dem_pos,hou_dem_cue,hou_dem_inc)
housevote_SD <- matrix(rep(NA,ncol(SD)*n),nrow=n)
for(j in 1:ncol(SD)) housevote_SD[1:n,j] <- SD[1:n,j] * housevote
instr_SD <- matrix(rep(NA,ncol(SD)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SD)) for(k in 1:ncol(instr)) instr_SD[1:n,ncol(SD)*(k-1)+j] <- instr[1:n,k] * SD[1:n,j]
y <- hou_dem_place
S <- SD
C <- housevote
Z <- cbind(voter_pos,hou_med_voter,SD,instr_hou_voter_pos,instr_hou_med_voter,instr_SD)
gmms[[1]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1)
gmms[[2]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1&highknow)
gmms[[3]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1&lowknow)

SR <- cbind(1,hou_rep_pos,hou_rep_cue,-hou_rep_inc)
housevote_SR <- matrix(rep(NA,ncol(SR)*n),nrow=n)
for(j in 1:ncol(SR)) housevote_SR[1:n,j] <- SR[1:n,j] * housevote
instr_SR <- matrix(rep(NA,ncol(SR)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SR)) for(k in 1:ncol(instr)) instr_SR[1:n,ncol(SR)*(k-1)+j] <- instr[1:n,k] * SR[1:n,j]
y <- hou_rep_place
S <- SR
C <- 1 - housevote
Z <- cbind(voter_pos,hou_med_voter,SR,instr_hou_voter_pos,instr_hou_med_voter,instr_SR)
gmms[[4]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1)
gmms[[5]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1&highknow)
gmms[[6]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1&lowknow)

mat4 <- matrix(rep(NA,23*6),ncol=6)
for(i in 1:6)
{
	mat4[c(1,3,5,7,9,11,13,15),i] <- as.coef(gmms[[i]]$theta,gmms[[i]]$thetase)
	mat4[c(2,4,6,8,10,12,14,16),i] <- as.se(gmms[[i]]$thetase)

	mat4[17,i] <- gmms[[i]]$N
	mat4[18,i] <- gmms[[i]]$Clusters
	mat4[19,i] <- dig1(gmms[[i]]$firststage$fstatistic[1])
	
	dm <- delta.method(wald1234_1,gmms[[i]]$theta,gmms[[i]]$V)
	mat4[20:23,i] <- as.pval(dm$est,dm$se)	
}
mat4labels <- c("$\\alpha$ (assimilation)","","$\\gamma$ (contrast)","","$\\delta$ (cheerleading)","","$\\tau$ (anti-cheerleading)","","$\\lambda_1$ (Constant)","","$\\lambda_2$ (Position)","","$\\lambda_3$ (Cue)","","$\\lambda_4$ (Incumbent)","","N","Clusters","First Stage F-Statistic","p-value for $H_0: \\alpha_k = 1$","p-value for $H_0: \\gamma_k = 1$","p-value for $H_0: \\delta_k = 1$","p-value for $H_0: \\tau_k = 1$")

# table 6 # 
reldist <- (hou_rep_place - voter_pos)^2 - (hou_dem_place - voter_pos)^2
f1 <- housevote ~ reldist + hou_dem_chq + hou_rep_chq + stunem + stunem_party_gov + hou_dem_inc + hou_rep_inc | hou_dem_chq + hou_rep_chq + stunem + stunem_party_gov + hou_dem_inc + hou_rep_inc + hou_dem_cue + hou_rep_cue + hou_dem_pos + hou_rep_pos + voter_pos
ivs <- list()
ivs[[1]] <- ivreg0(f1,se="cluster",unitid=statecd,subset=samp1)
ivs[[2]] <- ivreg0(f1,se="cluster",unitid=statecd,subset=samp1&highknow)
ivs[[3]] <- ivreg0(f1,se="cluster",unitid=statecd,subset=samp1&lowknow)

mat5 <- matrix(rep(NA,19*3),ncol=3)
for(i in 1:3)
{
	mat5[c(1,3,5,7,9,11,13,15),i] <- as.coef(ivs[[i]]$coef,ivs[[i]]$se)
	mat5[c(2,4,6,8,10,12,14,16),i] <- as.se(ivs[[i]]$se)

	mat5[17,i] <- ivs[[i]]$N
	mat5[18,i] <- ivs[[i]]$Clusters
	mat5[19,i] <- dig1(ivs[[i]]$FirstStageF)
}
mat5labels <- c("Intercept","","Perceived Policy Distance","","Democratic Challenger Quality","","Republican Challenger Quality","","Unemployment","","Unemployment * Party of Governor","","Democratic Incumbent","","Republican Incumbent","","N","Clusters","First Stage F-Statistic")

# table 7 # 
mat9 <- matrix(rep(NA,8*6),ncol=6)

# change perception
mat9[1,1] <- dig3(-mean0(ivs[[1]]$model[2]*((-1 - voter_pos)^2 - (-.5 - voter_pos)^2),subset=samp1))
mat9[1,3] <- dig3(-mean0(ivs[[2]]$model[2]*((-1 - voter_pos)^2 - (-.5 - voter_pos)^2),subset=samp1))
mat9[1,5] <- dig3(-mean0(ivs[[3]]$model[2]*((-1 - voter_pos)^2 - (-.5 - voter_pos)^2),subset=samp1))
mat9[1,2] <- dig3(-mean0(ivs[[1]]$model[2]*((-1 - voter_pos)^2 - (-1.5 - voter_pos)^2),subset=samp1))
mat9[1,4] <- dig3(-mean0(ivs[[2]]$model[2]*((-1 - voter_pos)^2 - (-1.5 - voter_pos)^2),subset=samp1))
mat9[1,6] <- dig3(-mean0(ivs[[3]]$model[2]*((-1 - voter_pos)^2 - (-1.5 - voter_pos)^2),subset=samp1))
mat9[5,2] <- dig3(-mean0(ivs[[1]]$model[2]*((1 - voter_pos)^2 - (.5 - voter_pos)^2),subset=samp1))
mat9[5,4] <- dig3(-mean0(ivs[[2]]$model[2]*((1 - voter_pos)^2 - (.5 - voter_pos)^2),subset=samp1))
mat9[5,6] <- dig3(-mean0(ivs[[3]]$model[2]*((1 - voter_pos)^2 - (.5 - voter_pos)^2),subset=samp1))
mat9[5,1] <- dig3(-mean0(ivs[[1]]$model[2]*((1 - voter_pos)^2 - (1.5 - voter_pos)^2),subset=samp1))
mat9[5,3] <- dig3(-mean0(ivs[[2]]$model[2]*((1 - voter_pos)^2 - (1.5 - voter_pos)^2),subset=samp1))
mat9[5,5] <- dig3(-mean0(ivs[[3]]$model[2]*((1 - voter_pos)^2 - (1.5 - voter_pos)^2),subset=samp1))

# change position
mat9[2,1] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) + .5 * gmms[[1]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[2,3] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) + .5 * gmms[[2]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[2,5] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) + .5 * gmms[[3]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[2,2] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) - .5 * gmms[[1]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[2,4] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) - .5 * gmms[[2]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[2,6] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) - .5 * gmms[[3]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[6,2] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) - .5 * gmms[[4]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[6,4] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) - .5 * gmms[[5]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[6,6] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) - .5 * gmms[[6]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[6,1] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) + .5 * gmms[[4]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[6,3] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) + .5 * gmms[[5]]$theta[6] - voter_pos)^2),subset=samp1))
mat9[6,5] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) + .5 * gmms[[6]]$theta[6] - voter_pos)^2),subset=samp1))

# change cue
mat9[3,1] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) + .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[3,3] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) + .5 * gmms[[2]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[3,5] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) + .5 * gmms[[3]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[3,2] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) - .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[3,4] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) - .5 * gmms[[2]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[3,6] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) - .5 * gmms[[3]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[7,2] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) - .5 * gmms[[4]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[7,4] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) - .5 * gmms[[5]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[7,6] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) - .5 * gmms[[6]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[7,1] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) + .5 * gmms[[4]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[7,3] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) + .5 * gmms[[5]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[7,5] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) + .5 * gmms[[6]]$theta[7] - voter_pos)^2),subset=samp1))

# change position and cue
mat9[4,1] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) + .5 * gmms[[1]]$theta[6] + .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[4,3] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) + .5 * gmms[[2]]$theta[6] + .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[4,5] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) + .5 * gmms[[3]]$theta[6] + .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[4,2] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) - .5 * gmms[[1]]$theta[6] - .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[4,4] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) - .5 * gmms[[2]]$theta[6] - .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[4,6] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_dem_place,subset=samp1) - voter_pos)^2 - (mean0(hou_dem_place,subset=samp1) - .5 * gmms[[3]]$theta[6] - .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[8,2] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) - .5 * gmms[[4]]$theta[6] - .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[8,4] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) - .5 * gmms[[5]]$theta[6] - .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[8,6] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) - .5 * gmms[[6]]$theta[6] - .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[8,1] <- dig3(-mean0(ivs[[1]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) + .5 * gmms[[4]]$theta[6] + .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[8,3] <- dig3(-mean0(ivs[[2]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) + .5 * gmms[[5]]$theta[6] + .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9[8,5] <- dig3(-mean0(ivs[[3]]$model[2]*((mean0(hou_rep_place,subset=samp1) - voter_pos)^2 - (mean0(hou_rep_place,subset=samp1) + .5 * gmms[[6]]$theta[6] + .5 * gmms[[1]]$theta[7] - voter_pos)^2),subset=samp1))
mat9labels <- c("Change Perceptions","Change Position","Change Cue","Change Position and Cue")

# figure 4 #
matH <- matrix(rep(NA,2*10),nrow=10)
matS <- matrix(rep(NA,2*10),nrow=10)
matV <- matrix(rep(NA,3*10),nrow=10)
for(i in 1:10)
{
	matH[i,1] <- sum(subset(YB[1:127,i],party[1:127]=="D") == 2) / sum(subset(YB[1:127,i],party[1:127]=="D") > 0) 
	matH[i,2] <- sum(subset(YB[1:127,i],party[1:127]=="R") == 2) / sum(subset(YB[1:127,i],party[1:127]=="R") > 0) 

	matS[i,1] <- sum(subset(YB[128:678,i],party[128:678]=="D") == 2) / sum(subset(YB[128:678,i],party[128:678]=="D") > 0) 
	matS[i,2] <- sum(subset(YB[128:678,i],party[128:678]=="R") == 2) / sum(subset(YB[128:678,i],party[128:678]=="R") > 0) 
	
	matV[i,1] <- sum(subset(YB[679:55213,i],party[679:55213]=="D") == 2) / sum(subset(YB[679:55213,i],party[679:55213]=="D") > 0) 
	matV[i,2] <- sum(subset(YB[679:55213,i],party[679:55213]=="R") == 2) / sum(subset(YB[679:55213,i],party[679:55213]=="R") > 0) 
	matV[i,3] <- sum(subset(YB[679:55213,i],party[679:55213]=="I") == 2) / sum(subset(YB[679:55213,i],party[679:55213]=="I") > 0) 
}

pdf(file=paste(dir1,"fig4.pdf",sep=""),width=12,height=6)
plot(0:1,type='n',ylim=c(0,11),xlim=c(-1,1),ylab="Bridge Vote",xlab="Percent Voting Yea", yaxt="n", xaxt="n")
axis(2, at=1:10,labels=10:1,las=2)
axis(1, at=.2*(0:5),labels=.2*(0:5),las=1)
text(matV[1:10,1],11-(1:10),"V",col='blue')
text(matV[1:10,2],11-(1:10),"V",col='red')
text(matV[1:10,3],11-(1:10),"V",col='black')
text(matH[1:10,1],11-(1:10),"H",col='blue')
text(matH[1:10,2],11-(1:10),"H",col='red')
text(matS[1:10,1],11-(1:10),"S",col='blue')
text(matS[1:10,2],11-(1:10),"S",col='red')
text(-.5,11-(1:10),bridges)
dev.off()

# table 8 #
libcon <- (YB[,1] == 2) + (YB[,2] == 2) + (YB[,3] == 1) + (YB[,4] == 1) + (YB[,5] == 1) + (YB[,6] == 2) + (YB[,7] == 2) + (YB[,8] == 2) + (YB[,10] == 2)
libconsum <- (YB[,1] > 0) + (YB[,2] > 0) + (YB[,3] > 0) + (YB[,4] > 0) + (YB[,5] > 0) + (YB[,6] > 0) + (YB[,7] > 0) + (YB[,8] > 0) + (YB[,10] > 0)
libcon <- libcon / libconsum
libcon <- 2 * libcon - 1
TotBridge <- rowSums(YB > 0)
for(i in 1:length(libcon)) {
	if(TotBridge[i] == 0) {
		libcon[i] <- NA
	}
}

mat14 <- matrix(rep(NA,7),ncol=1)
mat14[1,] <- dig3(mean0(libcon,subset=chamberparty=="SD"))
mat14[2,] <- dig3(mean0(libcon,subset=chamberparty=="HD"))
mat14[3,] <- dig3(mean0(libcon,subset=chamberparty=="VD"))
mat14[4,] <- dig3(mean0(libcon,subset=chamber=="V"))
mat14[5,] <- dig3(mean0(libcon,subset=chamberparty=="VR"))
mat14[6,] <- dig3(mean0(libcon,subset=chamberparty=="SR"))
mat14[7,] <- dig3(mean0(libcon,subset=chamberparty=="HR"))
mat14labels <- c("Mean Senate Democrat","Mean House Democrat","Mean Democratic Identifier","Mean Respondent","Mean Republican Identifier","Mean Senate Republican","Mean House Republican")

# robustness tests #

# table 9 #
instr <- cbind(hou_dem_chq,hou_rep_chq,stunem,stunem_party_gov)
instr_hou_voter_pos <- matrix(rep(NA,ncol(instr)*n),nrow=n)
instr_hou_med_voter <- matrix(rep(NA,ncol(instr)*n),nrow=n)
for(j in 1:ncol(instr))
{
	instr_hou_voter_pos[1:n,j] <- instr[1:n,j] * voter_pos
	instr_hou_med_voter[1:n,j] <- instr[1:n,j] * hou_med_voter
}
X <- voter_pos
M <- hou_med_voter
wald1234_1 <- function(x) { c(x[1] - 1,x[2] - 1,x[3] - 1,x[4] - 1) }
wald67 <- function(x) { x[6] + x[7] }
gmms4 <- list()
gmms4.pvals <- list()
gmms4.pvals4 <- matrix(rep(NA,12*4),ncol=12)

SD <- cbind(1,hou_dem_pos,hou_dem_cue)
housevote_SD <- matrix(rep(NA,ncol(SD)*n),nrow=n)
for(j in 1:ncol(SD)) housevote_SD[1:n,j] <- SD[1:n,j] * housevote
instr_SD <- matrix(rep(NA,ncol(SD)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SD)) for(k in 1:ncol(instr)) instr_SD[1:n,ncol(SD)*(k-1)+j] <- instr[1:n,k] * SD[1:n,j]
y <- hou_dem_place
S <- SD
C <- housevote
Z <- cbind(voter_pos,hou_med_voter,SD,instr_hou_voter_pos,instr_hou_med_voter,instr_SD)
gmms4[[1]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&hou_dem_inc==1)
gmms4[[2]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&highknow&hou_dem_inc==1)
gmms4[[3]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&lowknow&hou_dem_inc==1)
gmms4[[4]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&hou_dem_inc==0)
gmms4[[5]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&highknow&hou_dem_inc==0)
gmms4[[6]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&lowknow&hou_dem_inc==0)

SR <- cbind(1,hou_rep_pos,hou_rep_cue)
housevote_SR <- matrix(rep(NA,ncol(SR)*n),nrow=n)
for(j in 1:ncol(SR)) housevote_SR[1:n,j] <- SR[1:n,j] * housevote
instr_SR <- matrix(rep(NA,ncol(SR)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SR)) for(k in 1:ncol(instr)) instr_SR[1:n,ncol(SR)*(k-1)+j] <- instr[1:n,k] * SR[1:n,j]
y <- hou_rep_place
S <- SR
C <- 1 - housevote
Z <- cbind(voter_pos,hou_med_voter,SR,instr_hou_voter_pos,instr_hou_med_voter,instr_SR)
gmms4[[7]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&hou_rep_inc==1)
gmms4[[8]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&highknow&hou_rep_inc==1)
gmms4[[9]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&lowknow&hou_rep_inc==1)
gmms4[[10]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&hou_rep_inc==0)
gmms4[[11]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&highknow&hou_rep_inc==0)
gmms4[[12]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=5,cluster=statecd,subset=samp1&lowknow&hou_rep_inc==0)

mat8 <- matrix(rep(NA,17*12),ncol=12)
for(i in 1:12)
{
	mat8[c(1,3,5,7,9,11,13),i] <- as.coef(gmms4[[i]]$theta,gmms4[[i]]$thetase)
	mat8[c(2,4,6,8,10,12,14),i] <- as.se(gmms4[[i]]$thetase)

	mat8[15,i] <- gmms4[[i]]$N
	mat8[16,i] <- gmms4[[i]]$Clusters
	mat8[17,i] <- dig1(gmms4[[i]]$firststage$fstatistic[1])
}
mat8labels <- c("$\\alpha$ (assimilation)","","$\\gamma$ (contrast)","","$\\delta$ (cheerleading)","","$\\tau$ (anti-cheerleading)","","$\\lambda_1$ (Constant)","","$\\lambda_2$ (Position)","","$\\lambda_3$ (Cue)","","N","Clusters","First Stage F-Statistic")

# table 10 #
instr <- cbind(hou_dem_chq,hou_rep_chq,stunem,stunem_party_gov)
instr_hou_voter_pos <- matrix(rep(NA,ncol(instr)*n),nrow=n)
instr_hou_med_voter <- matrix(rep(NA,ncol(instr)*n),nrow=n)
for(j in 1:ncol(instr))
{
	instr_hou_voter_pos[1:n,j] <- instr[1:n,j] * voter_pos
	instr_hou_med_voter[1:n,j] <- instr[1:n,j] * hou_med_voter
}
X <- voter_pos
M <- hou_med_voter
wald1234_1 <- function(x) { c(x[1] - 1,x[2] - 1,x[3] - 1,x[4] - 1) }
wald678 <- function(x) { x[6] + x[7] + x[8] }
gmms5 <- list()
gmms5.pvals <- list()
gmms5.pvals2 <- matrix(rep(NA,6*4),ncol=6)

SD <- cbind(1,hou_dem_pos,hou_dem_cue,hou_dem_pos_old_npat,hou_dem_inc)
housevote_SD <- matrix(rep(NA,ncol(SD)*n),nrow=n)
for(j in 1:ncol(SD)) housevote_SD[1:n,j] <- SD[1:n,j] * housevote
instr_SD <- matrix(rep(NA,ncol(SD)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SD)) for(k in 1:ncol(instr)) instr_SD[1:n,ncol(SD)*(k-1)+j] <- instr[1:n,k] * SD[1:n,j]
y <- hou_dem_place
S <- SD
C <- housevote
Z <- cbind(voter_pos,hou_med_voter,SD,instr_hou_voter_pos,instr_hou_med_voter,instr_SD)
gmms5[[1]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=7,cluster=statecd,subset=samp1)
gmms5[[2]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=7,cluster=statecd,subset=samp1&highknow)
gmms5[[3]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=7,cluster=statecd,subset=samp1&lowknow)

SR <- cbind(1,hou_rep_pos,hou_rep_cue,hou_rep_pos_old_npat,-hou_rep_inc)
housevote_SR <- matrix(rep(NA,ncol(SR)*n),nrow=n)
for(j in 1:ncol(SR)) housevote_SR[1:n,j] <- SR[1:n,j] * housevote
instr_SR <- matrix(rep(NA,ncol(SR)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SR)) for(k in 1:ncol(instr)) instr_SR[1:n,ncol(SR)*(k-1)+j] <- instr[1:n,k] * SR[1:n,j]
y <- hou_rep_place
S <- SR
C <- 1 - housevote
Z <- cbind(voter_pos,hou_med_voter,SR,instr_hou_voter_pos,instr_hou_med_voter,instr_SR)
gmms5[[4]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=7,cluster=statecd,subset=samp1)
gmms5[[5]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=7,cluster=statecd,subset=samp1&highknow)
gmms5[[6]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=7,cluster=statecd,subset=samp1&lowknow)

mat10 <- matrix(rep(NA,25*6),ncol=6)
for(i in 1:6)
{
	mat10[c(1,3,5,7,9,11,13,15,17),i] <- as.coef(gmms5[[i]]$theta,gmms5[[i]]$thetase)
	mat10[c(2,4,6,8,10,12,14,16,18),i] <- as.se(gmms5[[i]]$thetase)

	mat10[19,i] <- gmms5[[i]]$N
	mat10[20,i] <- gmms5[[i]]$Clusters
	mat10[21,i] <- dig1(gmms5[[i]]$firststage$fstatistic[1])
	
	dm <- delta.method(wald1234_1,gmms5[[i]]$theta,gmms5[[i]]$V)
	mat10[22:25,i] <- as.pval(dm$est,dm$se)	
}
mat10labels <- c("$\\alpha$ (assimilation)","","$\\gamma$ (contrast)","","$\\delta$ (cheerleading)","","$\\tau$ (anti-cheerleading)","","$\\lambda_1$ (Constant)","","$\\lambda_2$ (Position)","","$\\lambda_3$ (Cue from statewide candidates)","","$\\lambda_4$ (Cue from previous candidate)","","$\\lambda_5$ (Incumbent)","","N","Clusters","First Stage F-Statistic","p-value for $H_0: \\alpha_k = 1$","p-value for $H_0: \\gamma_k = 1$","p-value for $H_0: \\delta_k = 1$","p-value for $H_0: \\tau_k = 1$")

# table 11 #
hou_dem_pos <- hou_dem_pos_2
hou_rep_pos <- hou_rep_pos_2
hou_dem_cue <- hou_dem_cue_2
hou_rep_cue <- hou_rep_cue_2

instr <- cbind(hou_dem_chq,hou_rep_chq,stunem,stunem_party_gov)
instr_hou_voter_pos <- matrix(rep(NA,ncol(instr)*n),nrow=n)
instr_hou_med_voter <- matrix(rep(NA,ncol(instr)*n),nrow=n)
for(j in 1:ncol(instr))
{
	instr_hou_voter_pos[1:n,j] <- instr[1:n,j] * voter_pos
	instr_hou_med_voter[1:n,j] <- instr[1:n,j] * hou_med_voter
}
X <- voter_pos
M <- hou_med_voter
wald1234_1 <- function(x) { c(x[1] - 1,x[2] - 1,x[3] - 1,x[4] - 1) }
wald67 <- function(x) { x[6] + x[7] }
gmms <- list()

SD <- cbind(1,hou_dem_pos,hou_dem_cue,hou_dem_inc)
housevote_SD <- matrix(rep(NA,ncol(SD)*n),nrow=n)
for(j in 1:ncol(SD)) housevote_SD[1:n,j] <- SD[1:n,j] * housevote
instr_SD <- matrix(rep(NA,ncol(SD)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SD)) for(k in 1:ncol(instr)) instr_SD[1:n,ncol(SD)*(k-1)+j] <- instr[1:n,k] * SD[1:n,j]
y <- hou_dem_place
S <- SD
C <- housevote
Z <- cbind(voter_pos,hou_med_voter,SD,instr_hou_voter_pos,instr_hou_med_voter,instr_SD)
gmms[[1]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1)
gmms[[2]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1&highknow)
gmms[[3]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1&lowknow)

SR <- cbind(1,hou_rep_pos,hou_rep_cue,-hou_rep_inc)
housevote_SR <- matrix(rep(NA,ncol(SR)*n),nrow=n)
for(j in 1:ncol(SR)) housevote_SR[1:n,j] <- SR[1:n,j] * housevote
instr_SR <- matrix(rep(NA,ncol(SR)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SR)) for(k in 1:ncol(instr)) instr_SR[1:n,ncol(SR)*(k-1)+j] <- instr[1:n,k] * SR[1:n,j]
y <- hou_rep_place
S <- SR
C <- 1 - housevote
Z <- cbind(voter_pos,hou_med_voter,SR,instr_hou_voter_pos,instr_hou_med_voter,instr_SR)
gmms[[4]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1)
gmms[[5]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1&highknow)
gmms[[6]] <- ivgmm_run(y,X,M,S,C,Z,rep(1,n),opt=1,cluster=statecd,subset=samp1&lowknow)

mat11 <- matrix(rep(NA,23*6),ncol=6)
for(i in 1:6)
{
	mat11[c(1,3,5,7,9,11,13,15),i] <- as.coef(gmms[[i]]$theta,gmms[[i]]$thetase)
	mat11[c(2,4,6,8,10,12,14,16),i] <- as.se(gmms[[i]]$thetase)

	mat11[17,i] <- gmms[[i]]$N
	mat11[18,i] <- gmms[[i]]$Clusters
	mat11[19,i] <- dig1(gmms[[i]]$firststage$fstatistic[1])
	
	dm <- delta.method(wald1234_1,gmms[[i]]$theta,gmms[[i]]$V)
	mat11[20:23,i] <- as.pval(dm$est,dm$se)	
}
mat11labels <- c("$\\alpha$ (assimilation)","","$\\gamma$ (contrast)","","$\\delta$ (cheerleading)","","$\\tau$ (anti-cheerleading)","","$\\lambda_1$ (Constant)","","$\\lambda_2$ (Position)","","$\\lambda_3$ (Cue)","","$\\lambda_4$ (Incumbent)","","N","Clusters","First Stage F-Statistic","p-value for $H_0: \\alpha_k = 1$","p-value for $H_0: \\gamma_k = 1$","p-value for $H_0: \\delta_k = 1$","p-value for $H_0: \\tau_k = 1$")

# put bonica estimates back in
hou_dem_pos <- hou_dem_pos_3
hou_rep_pos <- hou_rep_pos_3
hou_dem_cue <- hou_dem_cue_3
hou_rep_cue <- hou_rep_cue_3

# table 12 #
instr <- cbind(hou_dem_chq,hou_rep_chq,stunem,stunem_party_gov)
instr_hou_voter_pos <- matrix(rep(NA,ncol(instr)*n),nrow=n)
instr_hou_med_voter <- matrix(rep(NA,ncol(instr)*n),nrow=n)
for(j in 1:ncol(instr))
{
	instr_hou_voter_pos[1:n,j] <- instr[1:n,j] * voter_pos
	instr_hou_med_voter[1:n,j] <- instr[1:n,j] * hou_med_voter
}
X <- voter_pos
M <- hou_med_voter
wald1234_1 <- function(x) { c(x[1] - 1,x[2] - 1,x[3] - 1,x[4] - 1) }
wald67 <- function(x) { x[6] + x[7] }
gmms <- list()

SD <- cbind(1,hou_dem_pos,hou_dem_cue,hou_dem_inc)
housevote_SD <- matrix(rep(NA,ncol(SD)*n),nrow=n)
for(j in 1:ncol(SD)) housevote_SD[1:n,j] <- SD[1:n,j] * housevote
instr_SD <- matrix(rep(NA,ncol(SD)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SD)) for(k in 1:ncol(instr)) instr_SD[1:n,ncol(SD)*(k-1)+j] <- instr[1:n,k] * SD[1:n,j]
y <- hou_dem_place
S <- SD
C <- housevote
Z <- cbind(voter_pos,hou_med_voter,SD,instr_hou_voter_pos,instr_hou_med_voter,instr_SD)
gmms[[1]] <- ivgmm_run(y,X,M,S,C,Z,w1,opt=1,cluster=statecd,subset=samp1)
gmms[[2]] <- ivgmm_run(y,X,M,S,C,Z,w1,opt=1,cluster=statecd,subset=samp1&highknow)
gmms[[3]] <- ivgmm_run(y,X,M,S,C,Z,w1,opt=1,cluster=statecd,subset=samp1&lowknow)

SR <- cbind(1,hou_rep_pos,hou_rep_cue,-hou_rep_inc)
housevote_SR <- matrix(rep(NA,ncol(SR)*n),nrow=n)
for(j in 1:ncol(SR)) housevote_SR[1:n,j] <- SR[1:n,j] * housevote
instr_SR <- matrix(rep(NA,ncol(SR)*ncol(instr)*n),nrow=n)
for(j in 1:ncol(SR)) for(k in 1:ncol(instr)) instr_SR[1:n,ncol(SR)*(k-1)+j] <- instr[1:n,k] * SR[1:n,j]
y <- hou_rep_place
S <- SR
C <- 1 - housevote
Z <- cbind(voter_pos,hou_med_voter,SR,instr_hou_voter_pos,instr_hou_med_voter,instr_SR)
gmms[[4]] <- ivgmm_run(y,X,M,S,C,Z,w1,opt=1,cluster=statecd,subset=samp1)
gmms[[5]] <- ivgmm_run(y,X,M,S,C,Z,w1,opt=1,cluster=statecd,subset=samp1&highknow)
gmms[[6]] <- ivgmm_run(y,X,M,S,C,Z,w1,opt=1,cluster=statecd,subset=samp1&lowknow)

mat12 <- matrix(rep(NA,23*6),ncol=6)
for(i in 1:6)
{
	mat12[c(1,3,5,7,9,11,13,15),i] <- as.coef(gmms[[i]]$theta,gmms[[i]]$thetase)
	mat12[c(2,4,6,8,10,12,14,16),i] <- as.se(gmms[[i]]$thetase)

	mat12[17,i] <- gmms[[i]]$N
	mat12[18,i] <- gmms[[i]]$Clusters
	mat12[19,i] <- dig1(gmms[[i]]$firststage$fstatistic[1])
	
	dm <- delta.method(wald1234_1,gmms[[i]]$theta,gmms[[i]]$V)
	mat12[20:23,i] <- as.pval(dm$est,dm$se)	
}
mat12labels <- c("$\\alpha$ (assimilation)","","$\\gamma$ (contrast)","","$\\delta$ (cheerleading)","","$\\tau$ (anti-cheerleading)","","$\\lambda_1$ (Constant)","","$\\lambda_2$ (Position)","","$\\lambda_3$ (Cue)","","$\\lambda_4$ (Incumbent)","","N","Clusters","First Stage F-Statistic","p-value for $H_0: \\alpha_k = 1$","p-value for $H_0: \\gamma_k = 1$","p-value for $H_0: \\delta_k = 1$","p-value for $H_0: \\tau_k = 1$")

# table 13 #
reldist <- (hou_rep_place - voter_pos)^2 - (hou_dem_place - voter_pos)^2
f1 <- housevote ~ reldist + hou_dem_chq + hou_rep_chq + stunem + stunem_party_gov + hou_dem_inc + hou_rep_inc | hou_dem_chq + hou_rep_chq + stunem + stunem_party_gov + hou_dem_inc + hou_rep_inc + hou_dem_cue + hou_rep_cue + hou_dem_pos + hou_rep_pos + voter_pos
ivs <- list()
ivs[[1]] <- ivreg0(f1,se="cluster",unitid=statecd,subset=samp1,w=w1)
ivs[[2]] <- ivreg0(f1,se="cluster",unitid=statecd,subset=samp1&highknow,w=w1)
ivs[[3]] <- ivreg0(f1,se="cluster",unitid=statecd,subset=samp1&lowknow,w=w1)

mat13 <- matrix(rep(NA,19*3),ncol=3)
for(i in 1:3)
{
	mat13[c(1,3,5,7,9,11,13,15),i] <- as.coef(ivs[[i]]$coef,ivs[[i]]$se)
	mat13[c(2,4,6,8,10,12,14,16),i] <- as.se(ivs[[i]]$se)

	mat13[17,i] <- ivs[[i]]$N
	mat13[18,i] <- ivs[[i]]$Clusters
	mat13[19,i] <- dig1(ivs[[i]]$FirstStageF)
}
mat13labels <- c("Intercept","","Perceived Policy Distance","","Democratic Challenger Quality","","Republican Challenger Quality","","Unemployment","","Unemployment * Party of Governor","","Democratic Incumbent","","Republican Incumbent","","N","Clusters","First Stage F-Statistic")

# generate latex tables #
file1 <- paste(dir1,"tables.tex",sep="")
latex_doc_start(file1)

# table 1 #
latex_table_start(file1,1)
latex_table_entries(file1,labels1[1:7],matrix(vec1[1:7,],ncol=1))
latex_add_line(file1,"\\\\")
latex_table_entries(file1,labels1[8:11],matrix(vec1[8:11,],ncol=1))
latex_add_line(file1,"\\\\")
latex_table_entries(file1,labels1[12:16],matrix(vec1[12:16,],ncol=1))
latex_table_end(file1,label="HouseSummaryTable",caption="Summary Statistics for Ideal Point Estimates.")

# table 2 #
latex_table_start(file1,4)
latex_add_line(file1," & \\multicolumn{2}{c}{\\textbf{Democratic Placements}} & \\multicolumn{2}{c}{\\textbf{Republican Placements}} \\\\")
latex_add_line(file1," & Assimilation & Contrast & Assimilation & Contrast \\\\")
latex_add_line(file1," & (Dem. Voters) & (Rep. Voters) & (Rep. Voters) & (Dem. Voters) \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,c("$\\alpha$","","$\\gamma$",""),mat1[1:4,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,c("N","Clusters"),mat1[5:6,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,c("p-value for $H_0: \\alpha=1$","p-value for $H_0: \\gamma=1$"),mat1[7:8,])
latex_table_end(file1,label="AssimTable",caption="Test of the Assimilation and Contrast Theory. Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

# table 3 #
latex_table_start(file1,4)
latex_add_line(file1," & \\multicolumn{2}{c}{\\textbf{Democratic Placements}} & \\multicolumn{2}{c}{\\textbf{Republican Placements}} \\\\")
latex_add_line(file1," & Cheerleading & Anti-cheerleading & Cheerleading & Anti-cheerleading \\\\")
latex_add_line(file1," & (Dem. Voters) & (Rep. Voters) & (Rep. Voters) & (Dem. Voters) \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,c("$\\delta$","","$\\tau$",""),mat2[1:4,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,c("N","Clusters"),mat2[5:6,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,c("p-value for $H_0: \\delta=1$","p-value for $H_0: \\tau=1$"),mat2[7:8,])
latex_table_end(file1,label="CheerleadingTable",caption="Test of the Cheerleading Theory. Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

# table 4 #
latex_table_start(file1,6)
latex_add_line(file1," & \\multicolumn{3}{c}{\\textbf{Democratic Placements}} & \\multicolumn{3}{c}{\\textbf{Republican Placements}} \\\\")
latex_add_line(file1," & All & High Know. & Low Know. & All & High Know. & Low Know. \\\\")
latex_add_line(file1," & Respondents & Respondents & Respondents & Respondents & Respondents & Respondents \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,c("Constant","","Candidate Position","","Cue from Higher Office Candidates","","Incumbent",""),mat3[1:8,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,c("N","Clusters","$R^2$",""),mat3[9:12,])
latex_table_end(file1,label="InformationTable",caption="Information Theory Test. Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

# table 5 #
latex_table_start(file1,6)
latex_add_line(file1," & \\multicolumn{3}{c}{\\textbf{Democratic Placements}} & \\multicolumn{3}{c}{\\textbf{Republican Placements}} \\\\")
latex_add_line(file1," & All & High Know. & Low Know. & All & High Know. & Low Know. \\\\")
latex_add_line(file1," & Respondents & Respondents & Respondents & Respondents & Respondents & Respondents \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat4labels[1:16],mat4[1:16,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat4labels[17:19],mat4[17:19,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat4labels[20:23],mat4[20:23,])
latex_table_end(file1,label="PlacementIVTable",caption="Instrumental Variable Estimates for Candidate Placement Equation. Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

# table 6 #
latex_table_start(file1,3)
latex_add_line(file1," & All & High Know. & Low Know. \\\\")
latex_add_line(file1," & Respondents & Respondents & Respondents \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat5labels[1:16],mat5[1:16,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat5labels[17:19],mat5[17:19,])
latex_table_end(file1,label="ChoiceIVTable",caption="Instrumental Variable Estimates for Candidate Choice. Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

# table 7 #
latex_table_start(file1,6)
latex_add_line(file1," & \\multicolumn{6}{c}{\\textbf{Democratic Placements}} \\\\")
latex_add_line(file1," & \\multicolumn{2}{c}{All} & \\multicolumn{2}{c}{High Know.} & \\multicolumn{2}{c}{Low Know.} \\\\")
latex_add_line(file1," & \\multicolumn{2}{c}{Respondents} & \\multicolumn{2}{c}{Respondents} & \\multicolumn{2}{c}{Respondents} \\\\")
latex_add_line(file1," & \\textit{Move Left} & \\textit{Move Right} & \\textit{Move Left} & \\textit{Move Right} & \\textit{Move Left} & \\textit{Move Right} \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat9labels,mat9[1:4,])
latex_add_line(file1,"\\\\")
latex_add_line(file1," & \\multicolumn{6}{c}{\\textbf{Republican Placements}} \\\\")
latex_add_line(file1," & \\multicolumn{2}{c}{All} & \\multicolumn{2}{c}{High Know.} & \\multicolumn{2}{c}{Low Know.} \\\\")
latex_add_line(file1," & \\multicolumn{2}{c}{Respondents} & \\multicolumn{2}{c}{Respondents} & \\multicolumn{2}{c}{Respondents} \\\\")
latex_add_line(file1," & \\textit{Move Left} & \\textit{Move Right} & \\textit{Move Left} & \\textit{Move Right} & \\textit{Move Left} & \\textit{Move Right} \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat9labels,mat9[5:8,])
latex_table_end(file1,label="METable",caption="Marginal Effects of Changes in House Candidate Positions and Statewide Candidate Positions on the Probability of Voting for the House Candidate.")

# table 8 #
latex_table_start(file1,1)
latex_table_entries(file1,mat14labels,mat14)
latex_table_end(file1,label="LibConTable",caption="Summary Statistics for Liberal-Conservative Index.")

# table 9 #
latex_table_start(file1,6)
latex_add_line(file1," & \\multicolumn{6}{c}{\\textbf{Democratic Placements}} \\\\")
latex_add_line(file1," & \\multicolumn{3}{c}{\\textbf{Democratic Incumbents}}& \\multicolumn{3}{c}{\\textbf{Democratic Challengers}} \\\\")
latex_add_line(file1," & All & High Know. & Low Know. & All & High Know. & Low Know. \\\\")
latex_add_line(file1," & Respondents & Respondents & Respondents & Respondents & Respondents & Respondents \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat8labels[1:14],mat8[1:14,1:6])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat8labels[15:17],mat8[15:17,1:6])
latex_table_end(file1,label="PlacementIVTableDemIncumb",caption="Instrumental Variable Estimates for Democratic Placements for Incumbents and Challengers. Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")
latex_table_start(file1,6)
latex_add_line(file1," & \\multicolumn{6}{c}{\\textbf{Republican Placements}} \\\\")
latex_add_line(file1," & \\multicolumn{3}{c}{\\textbf{Republican Incumbents}}& \\multicolumn{3}{c}{\\textbf{Republican Challengers}} \\\\")
latex_add_line(file1," & All & High Know. & Low Know. & All & High Know. & Low Know. \\\\")
latex_add_line(file1," & Respondents & Respondents & Respondents & Respondents & Respondents & Respondents \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat8labels[1:14],mat8[1:14,7:12])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat8labels[15:17],mat8[15:17,7:12])
latex_table_end(file1,label="PlacementIVTableRepIncumb",caption="Instrumental Variable Estimates for Republican Placements for Incumbent and Challengers. Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

# table 10 #
latex_table_start(file1,6)
latex_add_line(file1," & \\multicolumn{3}{c}{\\textbf{Democratic Placements}} & \\multicolumn{3}{c}{\\textbf{Republican Placements}} \\\\")
latex_add_line(file1," & All & High Know. & Low Know. & All & High Know. & Low Know. \\\\")
latex_add_line(file1," & Respondents & Respondents & Respondents & Respondents & Respondents & Respondents \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat10labels[1:18],mat10[1:18,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat10labels[19:21],mat10[19:21,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat10labels[22:25],mat10[22:25,])
latex_table_end(file1,label="PlacementIVTableOld",caption="Instrumental Variable Estimates for Candidate Placement Equation, Alternative Cue. Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

# table 11 #
latex_table_start(file1,6)
latex_add_line(file1," & \\multicolumn{3}{c}{\\textbf{Democratic Placements}} & \\multicolumn{3}{c}{\\textbf{Republican Placements}} \\\\")
latex_add_line(file1," & All & High Know. & Low Know. & All & High Know. & Low Know. \\\\")
latex_add_line(file1," & Respondents & Respondents & Respondents & Respondents & Respondents & Respondents \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat11labels[1:16],mat11[1:16,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat11labels[17:19],mat11[17:19,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat11labels[20:23],mat11[20:23,])
latex_table_end(file1,label="PlacementIVTableNoBonica",caption="Instrumental Variable Estimates for Candidate Placement Equation (no bonica imputations). Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

# table 12 #
latex_table_start(file1,6)
latex_add_line(file1," & \\multicolumn{3}{c}{\\textbf{Democratic Placements}} & \\multicolumn{3}{c}{\\textbf{Republican Placements}} \\\\")
latex_add_line(file1," & All & High Know. & Low Know. & All & High Know. & Low Know. \\\\")
latex_add_line(file1," & Respondents & Respondents & Respondents & Respondents & Respondents & Respondents \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat12labels[1:16],mat12[1:16,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat12labels[17:19],mat12[17:19,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat12labels[20:23],mat12[20:23,])
latex_table_end(file1,label="PlacementIVTableW",caption="Instrumental Variable Estimates for Candidate Placement Equation (weighted version). Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

# table 13 #
latex_table_start(file1,3)
latex_add_line(file1," & All & High Know. & Low Know. \\\\")
latex_add_line(file1," & Respondents & Respondents & Respondents \\\\")
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat13labels[1:16],mat13[1:16,])
latex_add_line(file1,"\\\\")
latex_table_entries(file1,mat13labels[17:19],mat13[17:19,])
latex_table_end(file1,label="ChoiceIVTableW",caption="Instrumental Variable Estimates for Candidate Choice (weighted version). Standard errors are clustered by congressional district. $^+ p<.10, ^{*} p<.05,  ^{**} p<.01, ^{***} p<.001$.")

latex_doc_end(file1)

# misc calculations reported in paper

# reported in section 5.2
quantile0(voter_pos,c(.2,.4,.6,.8),subset=samp1)
mean0(housevote,subset=voter_pos<quantile0(voter_pos,c(.2,.4,.6,.8),subset=samp1)[1]&samp1)
mean0(housevote,subset=voter_pos>quantile0(voter_pos,c(.2,.4,.6,.8),subset=samp1)[4]&samp1)

# reported in appendix A.1
median0(libcon,subset=party=="D"&chamber=="V")
quantile0(libcon,subset=party=="D"&chamber=="H",probs=.859)

median0(libcon,subset=party=="R"&chamber=="V")
quantile0(libcon,subset=party=="R"&chamber=="H",probs=1-.972)

dig3(mean0(selfplacement))
dig3(mean0(hou_dem_place0))
dig3(mean0(hou_rep_place0))
dig3(mean0(selfplacement) - mean0(hou_rep_place0))
dig3(mean0(hou_rep_place0) - mean0(selfplacement))
dig3(mean0(abs(hou_dem_place0 - selfplacement) < abs(hou_rep_place0 - selfplacement)))
dig3(mean0(abs(hou_dem_place0 - selfplacement) > abs(hou_rep_place0 - selfplacement)))
dig3(mean0(selfplacement,subset=dem==1))
dig3(mean0(selfplacement,subset=rep==1))




